import re
import time
from pyVmomi import vim, vmodl
from app.vmware_exp.examples import vcenter_utils


def extract_username(user):
    if "@" in user:
        username = user[0:user.lower().strip().find("@")]
    elif "\\" in user:
        username = user[user.lower().strip().find("\\") + 1:]
    else:
        username = user

    return username

def get_user_priviliges(full_user_name):
    try:

        si = vcenter_utils.connect_vcenter()
        content = si.RetrieveContent()
        auth_mgr = content.authorizationManager
        permissions = auth_mgr.RetrieveAllPermissions()
        user_name = extract_username(full_user_name)
        print ("Retrieving all the available permissions: {permissions}")
        for permission in permissions:
            user = permission.principal
            #print(f'........user: {user}')

            vc_user = extract_username(user)
            #print(f'........vc_user: {vc_user}')

            # Get the Role ID.
            if user_name.lower() == vc_user.lower():
                role_id = permission.roleId
                #print(f"User: {user_name}, Role ID: {role_id}")
                break

        # From the vCenter API we will get only the User/Group which is added
        # in the Global permissions ie. the permissions are given at
        # Group level not user level. So if user is a member of a group
        # and only group is added in global permissions then we need to
        # validate the user against the root folder.
        privileges = []
        if role_id:
            print("Retrieving all the available user roles...")
            user_roles = auth_mgr.roleList
            #print(f'........user roles: {user_roles}')

            for user_role in user_roles:
                if user_role.roleId == role_id:
                    privileges = user_role.privilege
                    print(f'........user role: {role_id}, privileges: {privileges}')
                    break
        else:
            mo_refs = [content.rootFolder] #[get_datacenters_root_folder()]
            privileges = \
                auth_mgr.FetchUserPrivilegeOnEntities(
                    entities=mo_refs, userName=user_name)
            print(f'........rootFolder: {mo_refs}, privileges: {privileges}')

    except Exception as error:
        raise error


if __name__ == '__main__':
    user = "administrator@vsphere.local"
    user = "admin2@vsphere.local"

    get_user_priviliges(user)


"""
C:\Python3\python.exe "C:/Users/aafakmoh/OneDrive - Hewlett Packard Enterprise/mypy/app/vmware_exp/examples/get_user_privileges.py"
connecting to vcenter...
connected, vcenter: %s 'vim.ServiceInstance:ServiceInstance'
Retrieving all the available permissions: {permissions}
........user: VSPHERE.LOCAL\admin2
........vc_user: admin2
User: admin2, Role ID: 621425699
Retrieving all the available user roles...
........user roles: (vim.AuthorizationManager.Role) [
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = -1233579335,
      system = false,
      name = 'com.vmware.Content.Admin',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'com.vmware.Content.Admin',
         summary = 'com.vmware.Content.Admin'
      },
      privilege = (str) [
         'ContentLibrary.AddLibraryItem',
         'ContentLibrary.AddSubscription',
         'ContentLibrary.CheckInTemplate',
         'ContentLibrary.CheckOutTemplate',
         'ContentLibrary.CreateLocalLibrary',
         'ContentLibrary.CreateSubscribedLibrary',
         'ContentLibrary.DeleteLibraryItem',
         'ContentLibrary.DeleteLocalLibrary',
         'ContentLibrary.DeleteSubscribedLibrary',
         'ContentLibrary.DeleteSubscription',
         'ContentLibrary.DownloadSession',
         'ContentLibrary.EvictLibraryItem',
         'ContentLibrary.EvictSubscribedLibrary',
         'ContentLibrary.GetConfiguration',
         'ContentLibrary.ManageRegistry',
         'ContentLibrary.ManageRegistryProject',
         'ContentLibrary.ProbeSubscription',
         'ContentLibrary.PublishLibrary',
         'ContentLibrary.PublishLibraryItem',
         'ContentLibrary.ReadStorage',
         'ContentLibrary.SyncLibrary',
         'ContentLibrary.SyncLibraryItem',
         'ContentLibrary.TypeIntrospection',
         'ContentLibrary.UpdateConfiguration',
         'ContentLibrary.UpdateLibrary',
         'ContentLibrary.UpdateLibraryItem',
         'ContentLibrary.UpdateLocalLibrary',
         'ContentLibrary.UpdateSession',
         'ContentLibrary.UpdateSubscribedLibrary',
         'ContentLibrary.UpdateSubscription',
         'System.Anonymous',
         'System.Read',
         'System.View'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = -8,
      system = true,
      name = 'NoTrustedAdmin',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'NoTrustedAdmin',
         summary = 'NoTrustedAdmin'
      },
      privilege = (str) [
         'Alarm.Acknowledge',
         'Alarm.Create',
         'Alarm.Delete',
         'Alarm.DisableActions',
         'Alarm.Edit',
         'Alarm.SetStatus',
         'Alarm.ToggleEnableOnEntity',
         'Authorization.ModifyPermissions',
         'Authorization.ModifyPrivileges',
         'Authorization.ModifyRoles',
         'Authorization.ReassignRolePermissions',
         'AutoDeploy.Host.AssociateMachine',
         'AutoDeploy.Profile.Create',
         'AutoDeploy.Profile.Edit',
         'AutoDeploy.Rule.Create',
         'AutoDeploy.Rule.Delete',
         'AutoDeploy.Rule.Edit',
         'AutoDeploy.RuleSet.Activate',
         'AutoDeploy.RuleSet.Edit',
         'Certificate.Manage',
         'CertificateAuthority.Administer',
         'CertificateAuthority.Manage',
         'CertificateManagement.Administer',
         'CertificateManagement.Manage',
         'Cns.Searchable',
         'ComputePolicy.Manage',
         'ContentLibrary.AddLibraryItem',
         'ContentLibrary.AddSubscription',
         'ContentLibrary.CheckInTemplate',
         'ContentLibrary.CheckOutTemplate',
         'ContentLibrary.CreateLocalLibrary',
         'ContentLibrary.CreateSubscribedLibrary',
         'ContentLibrary.DeleteLibraryItem',
         'ContentLibrary.DeleteLocalLibrary',
         'ContentLibrary.DeleteSubscribedLibrary',
         'ContentLibrary.DeleteSubscription',
         'ContentLibrary.DownloadSession',
         'ContentLibrary.EvictLibraryItem',
         'ContentLibrary.EvictSubscribedLibrary',
         'ContentLibrary.GetConfiguration',
         'ContentLibrary.ImportStorage',
         'ContentLibrary.ManageClusterRegistryResource',
         'ContentLibrary.ManageRegistry',
         'ContentLibrary.ManageRegistryProject',
         'ContentLibrary.ProbeSubscription',
         'ContentLibrary.PublishLibrary',
         'ContentLibrary.PublishLibraryItem',
         'ContentLibrary.ReadStorage',
         'ContentLibrary.SyncLibrary',
         'ContentLibrary.SyncLibraryItem',
         'ContentLibrary.TypeIntrospection',
         'ContentLibrary.UpdateConfiguration',
         'ContentLibrary.UpdateLibrary',
         'ContentLibrary.UpdateLibraryItem',
         'ContentLibrary.UpdateLocalLibrary',
         'ContentLibrary.UpdateSession',
         'ContentLibrary.UpdateSubscribedLibrary',
         'ContentLibrary.UpdateSubscription',
         'Cryptographer.Access',
         'Cryptographer.AddDisk',
         'Cryptographer.Clone',
         'Cryptographer.Decrypt',
         'Cryptographer.Encrypt',
         'Cryptographer.EncryptNew',
         'Cryptographer.ManageEncryptionPolicy',
         'Cryptographer.ManageKeyServers',
         'Cryptographer.ManageKeys',
         'Cryptographer.Migrate',
         'Cryptographer.Recrypt',
         'Cryptographer.RegisterHost',
         'Cryptographer.RegisterVM',
         'DVPortgroup.Create',
         'DVPortgroup.Delete',
         'DVPortgroup.Ipfix',
         'DVPortgroup.Modify',
         'DVPortgroup.PolicyOp',
         'DVPortgroup.ScopeOp',
         'DVSwitch.Create',
         'DVSwitch.Delete',
         'DVSwitch.HostOp',
         'DVSwitch.Ipfix',
         'DVSwitch.Modify',
         'DVSwitch.Move',
         'DVSwitch.PolicyOp',
         'DVSwitch.PortConfig',
         'DVSwitch.PortSetting',
         'DVSwitch.ResourceManagement',
         'DVSwitch.Vspan',
         'Datacenter.Create',
         'Datacenter.Delete',
         'Datacenter.IpPoolConfig',
         'Datacenter.IpPoolQueryAllocations',
         'Datacenter.IpPoolReleaseIp',
         'Datacenter.Move',
         'Datacenter.Reconfigure',
         'Datacenter.Rename',
         'Datastore.AllocateSpace',
         'Datastore.Browse',
         'Datastore.Config',
         'Datastore.Delete',
         'Datastore.DeleteFile',
         'Datastore.FileManagement',
         'Datastore.Move',
         'Datastore.Rename',
         'Datastore.UpdateVirtualMachineFiles',
         'Datastore.UpdateVirtualMachineMetadata',
         'EAM.Config',
         'EAM.Modify',
         'EAM.View',
         'Extension.Register',
         'Extension.Unregister',
         'Extension.Update',
         'ExternalStatsProvider.Register',
         'ExternalStatsProvider.Unregister',
         'ExternalStatsProvider.Update',
         'Folder.Create',
         'Folder.Delete',
         'Folder.Move',
         'Folder.Rename',
         'Global.CancelTask',
         'Global.CapacityPlanning',
         'Global.Diagnostics',
         'Global.DisableMethods',
         'Global.EnableMethods',
         'Global.GlobalTag',
         'Global.Health',
         'Global.Licenses',
         'Global.LogEvent',
         'Global.ManageCustomFields',
         'Global.Proxy',
         'Global.ScriptAction',
         'Global.ServiceManagers',
         'Global.SetCustomField',
         'Global.Settings',
         'Global.SystemTag',
         'Global.VCServer',
         'HLM.Manage',
         'HealthUpdateProvider.Register',
         'HealthUpdateProvider.Unregister',
         'HealthUpdateProvider.Update',
         'Host.Cim.CimInteraction',
         'Host.Config.AdvancedConfig',
         'Host.Config.AuthenticationStore',
         'Host.Config.AutoStart',
         'Host.Config.Connection',
         'Host.Config.DateTime',
         'Host.Config.Firmware',
         'Host.Config.GuestStore',
         'Host.Config.HyperThreading',
         'Host.Config.Image',
         'Host.Config.Maintenance',
         'Host.Config.Memory',
         'Host.Config.NetService',
         'Host.Config.Network',
         'Host.Config.Nvdimm',
         'Host.Config.Patch',
         'Host.Config.PciPassthru',
         'Host.Config.Power',
         'Host.Config.ProductLocker',
         'Host.Config.Quarantine',
         'Host.Config.Resources',
         'Host.Config.Settings',
         'Host.Config.Snmp',
         'Host.Config.Storage',
         'Host.Config.SystemManagement',
         'Host.Hbr.HbrManagement',
         'Host.Inventory.AddHostToCluster',
         'Host.Inventory.AddStandaloneHost',
         'Host.Inventory.CreateCluster',
         'Host.Inventory.DeleteCluster',
         'Host.Inventory.EditCluster',
         'Host.Inventory.ManageClusterLifecyle',
         'Host.Inventory.MoveCluster',
         'Host.Inventory.MoveHost',
         'Host.Inventory.RemoveHostFromCluster',
         'Host.Inventory.RenameCluster',
         'Host.Local.CreateVM',
         'Host.Local.DeleteVM',
         'Host.Local.InstallAgent',
         'Host.Local.ManageUserGroups',
         'Host.Local.ReconfigVM',
         'Infraprofile.Read',
         'Infraprofile.Write',
         'IntercomNamespace.Read',
         'IntercomNamespace.Write',
         'InventoryService.Tagging.AttachTag',
         'InventoryService.Tagging.CreateCategory',
         'InventoryService.Tagging.CreateTag',
         'InventoryService.Tagging.DeleteCategory',
         'InventoryService.Tagging.DeleteTag',
         'InventoryService.Tagging.EditCategory',
         'InventoryService.Tagging.EditTag',
         'InventoryService.Tagging.ModifyUsedByForCategory',
         'InventoryService.Tagging.ModifyUsedByForTag',
         'InventoryService.Tagging.ObjectAttachable',
         'NIMBLESTORAGE.Datacenter.com.nimblestorage.hi.View',
         'NIMBLESTORAGE.Datastore.Snapshot.com.nimblestorage.hi.Create',
         'NIMBLESTORAGE.Datastore.Snapshot.com.nimblestorage.hi.Delete',
         'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.Clone',
         'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.Create',
         'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.Delete',
         'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.Edit',
         'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.Grow',
         'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.View',
         'NIMBLESTORAGE.NimbleGroup.ManageGroup',
         'NIMBLESTORAGE.NimbleGroup.groupId.406793085223850253',
         'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.Claim',
         'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.ConfigureVss',
         'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.Delete',
         'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.Purge',
         'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.Restore',
         'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.Undelete',
         'NIMBLESTORAGE.VirtualMachine.View.com.nimblestorage.hi.Deleted',
         'NIMBLESTORAGE.VirtualMachine.View.com.nimblestorage.hi.Local',
         'NIMBLESTORAGE.VirtualMachine.View.com.nimblestorage.hi.Replicated',
         'Namespaces.Configure',
         'Namespaces.Manage',
         'Network.Assign',
         'Network.Config',
         'Network.Delete',
         'Network.Move',
         'Nsx.Manage',
         'Nsx.ModifyAll',
         'Nsx.Read',
         'Observability.Admin',
         'Performance.ModifyIntervals',
         'Plugin.Management',
         'Profile.Clear',
         'Profile.Create',
         'Profile.Delete',
         'Profile.Edit',
         'Profile.Export',
         'Profile.View',
         'Resource.ApplyRecommendation',
         'Resource.AssignVAppToPool',
         'Resource.AssignVMToPool',
         'Resource.ColdMigrate',
         'Resource.CreatePool',
         'Resource.DeletePool',
         'Resource.EditPool',
         'Resource.HotMigrate',
         'Resource.MovePool',
         'Resource.QueryVMotion',
         'Resource.RenamePool',
         'ScheduledTask.Create',
         'ScheduledTask.Delete',
         'ScheduledTask.Edit',
         'ScheduledTask.Run',
         'ServiceAccount.Administer',
         'ServiceAccount.ManageAccount',
         'ServiceAccount.ManagePassword',
         'Sessions.GlobalMessage',
         'Sessions.ImpersonateUser',
         'Sessions.TerminateSession',
         'Sessions.ValidateSession',
         'SettingsStore.Manage',
         'StoragePod.Config',
         'StorageProfile.Update',
         'StorageProfile.View',
         'StorageViews.ConfigureService',
         'StorageViews.View',
         'System.Anonymous',
         'System.Read',
         'System.View',
         'Task.Create',
         'Task.Update',
         'TenantManager.Query',
         'TenantManager.Update',
         'TransferService.Manage',
         'TransferService.Monitor',
         'Trust.Administer',
         'Trust.Manage',
         'VApp.ApplicationConfig',
         'VApp.AssignResourcePool',
         'VApp.AssignVApp',
         'VApp.AssignVM',
         'VApp.Clone',
         'VApp.Create',
         'VApp.Delete',
         'VApp.Export',
         'VApp.ExtractOvfEnvironment',
         'VApp.Import',
         'VApp.InstanceConfig',
         'VApp.ManagedByConfig',
         'VApp.Move',
         'VApp.PowerOff',
         'VApp.PowerOn',
         'VApp.PullFromUrls',
         'VApp.Rename',
         'VApp.ResourceConfig',
         'VApp.Suspend',
         'VApp.Unregister',
         'VcIdentityProviders.Create',
         'VcIdentityProviders.Manage',
         'VcIdentityProviders.Read',
         'VcIntegrity.Baseline.com.vmware.vcIntegrity.AssignBaselines',
         'VcIntegrity.Baseline.com.vmware.vcIntegrity.ManageBaselines',
         'VcIntegrity.FileUpload.com.vmware.vcIntegrity.ImportFile',
         'VcIntegrity.General.com.vmware.vcIntegrity.Configure',
         'VcIntegrity.HardwareCompatibility.Read',
         'VcIntegrity.Updates.com.vmware.vcIntegrity.Remediate',
         'VcIntegrity.Updates.com.vmware.vcIntegrity.Scan',
         'VcIntegrity.Updates.com.vmware.vcIntegrity.Stage',
         'VcIntegrity.Updates.com.vmware.vcIntegrity.ViewStatus',
         'VcIntegrity.lifecycleGeneral.Read',
         'VcIntegrity.lifecycleGeneral.Write',
         'VcIntegrity.lifecycleHealth.Read',
         'VcIntegrity.lifecycleHealth.Write',
         'VcIntegrity.lifecycleSettings.Read',
         'VcIntegrity.lifecycleSettings.Write',
         'VcIntegrity.lifecycleSoftwareRemediation.Read',
         'VcIntegrity.lifecycleSoftwareRemediation.Write',
         'VcIntegrity.lifecycleSoftwareSpecification.Read',
         'VcIntegrity.lifecycleSoftwareSpecification.Write',
         'VcLifecycle.View',
         'VirtualMachine.Config.AddExistingDisk',
         'VirtualMachine.Config.AddNewDisk',
         'VirtualMachine.Config.AddRemoveDevice',
         'VirtualMachine.Config.AdvancedConfig',
         'VirtualMachine.Config.Annotation',
         'VirtualMachine.Config.CPUCount',
         'VirtualMachine.Config.ChangeTracking',
         'VirtualMachine.Config.DiskExtend',
         'VirtualMachine.Config.DiskLease',
         'VirtualMachine.Config.EditDevice',
         'VirtualMachine.Config.HostUSBDevice',
         'VirtualMachine.Config.ManagedBy',
         'VirtualMachine.Config.Memory',
         'VirtualMachine.Config.MksControl',
         'VirtualMachine.Config.QueryFTCompatibility',
         'VirtualMachine.Config.QueryUnownedFiles',
         'VirtualMachine.Config.RawDevice',
         'VirtualMachine.Config.ReloadFromPath',
         'VirtualMachine.Config.RemoveDisk',
         'VirtualMachine.Config.Rename',
         'VirtualMachine.Config.ResetGuestInfo',
         'VirtualMachine.Config.Resource',
         'VirtualMachine.Config.Settings',
         'VirtualMachine.Config.SwapPlacement',
         'VirtualMachine.Config.ToggleForkParent',
         'VirtualMachine.Config.UpgradeVirtualHardware',
         'VirtualMachine.GuestOperations.Execute',
         'VirtualMachine.GuestOperations.Modify',
         'VirtualMachine.GuestOperations.ModifyAliases',
         'VirtualMachine.GuestOperations.Query',
         'VirtualMachine.GuestOperations.QueryAliases',
         'VirtualMachine.Hbr.ConfigureReplication',
         'VirtualMachine.Hbr.MonitorReplication',
         'VirtualMachine.Hbr.ReplicaManagement',
         'VirtualMachine.Interact.AnswerQuestion',
         'VirtualMachine.Interact.Backup',
         'VirtualMachine.Interact.ConsoleInteract',
         'VirtualMachine.Interact.CreateScreenshot',
         'VirtualMachine.Interact.CreateSecondary',
         'VirtualMachine.Interact.DefragmentAllDisks',
         'VirtualMachine.Interact.DeviceConnection',
         'VirtualMachine.Interact.DisableSecondary',
         'VirtualMachine.Interact.DnD',
         'VirtualMachine.Interact.EnableSecondary',
         'VirtualMachine.Interact.GuestControl',
         'VirtualMachine.Interact.MakePrimary',
         'VirtualMachine.Interact.Pause',
         'VirtualMachine.Interact.PowerOff',
         'VirtualMachine.Interact.PowerOn',
         'VirtualMachine.Interact.PutUsbScanCodes',
         'VirtualMachine.Interact.Record',
         'VirtualMachine.Interact.Replay',
         'VirtualMachine.Interact.Reset',
         'VirtualMachine.Interact.SESparseMaintenance',
         'VirtualMachine.Interact.SetCDMedia',
         'VirtualMachine.Interact.SetFloppyMedia',
         'VirtualMachine.Interact.Suspend',
         'VirtualMachine.Interact.SuspendToMemory',
         'VirtualMachine.Interact.TerminateFaultTolerantVM',
         'VirtualMachine.Interact.ToolsInstall',
         'VirtualMachine.Interact.TurnOffFaultTolerance',
         'VirtualMachine.Inventory.Create',
         'VirtualMachine.Inventory.CreateFromExisting',
         'VirtualMachine.Inventory.Delete',
         'VirtualMachine.Inventory.Move',
         'VirtualMachine.Inventory.Register',
         'VirtualMachine.Inventory.Unregister',
         'VirtualMachine.Namespace.Event',
         'VirtualMachine.Namespace.EventNotify',
         'VirtualMachine.Namespace.Management',
         'VirtualMachine.Namespace.ModifyContent',
         'VirtualMachine.Namespace.Query',
         'VirtualMachine.Namespace.ReadContent',
         'VirtualMachine.Provisioning.Clone',
         'VirtualMachine.Provisioning.CloneTemplate',
         'VirtualMachine.Provisioning.CreateTemplateFromVM',
         'VirtualMachine.Provisioning.Customize',
         'VirtualMachine.Provisioning.DeployTemplate',
         'VirtualMachine.Provisioning.DiskRandomAccess',
         'VirtualMachine.Provisioning.DiskRandomRead',
         'VirtualMachine.Provisioning.FileRandomAccess',
         'VirtualMachine.Provisioning.GetVmFiles',
         'VirtualMachine.Provisioning.MarkAsTemplate',
         'VirtualMachine.Provisioning.MarkAsVM',
         'VirtualMachine.Provisioning.ModifyCustSpecs',
         'VirtualMachine.Provisioning.PromoteDisks',
         'VirtualMachine.Provisioning.PutVmFiles',
         'VirtualMachine.Provisioning.ReadCustSpecs',
         'VirtualMachine.State.CreateSnapshot',
         'VirtualMachine.State.RemoveSnapshot',
         'VirtualMachine.State.RenameSnapshot',
         'VirtualMachine.State.RevertToSnapshot',
         'Vsan.Cluster.ShallowRekey',
         'vService.CreateDependency',
         'vService.DestroyDependency',
         'vService.ReconfigureDependency',
         'vService.UpdateDependency',
         'vStats.CollectAny',
         'vStats.QueryAny',
         'vStats.Settings'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = -7,
      system = true,
      name = 'TrustedAdmin',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'TrustedAdmin',
         summary = 'TrustedAdmin'
      },
      privilege = (str) [
         'System.Anonymous',
         'System.Read',
         'System.View',
         'TrustedAdmin.ConfigureHostCertificates',
         'TrustedAdmin.ConfigureHostMetadata',
         'TrustedAdmin.ConfigureTokenConversionPolicy',
         'TrustedAdmin.ManageAttestingSSO',
         'TrustedAdmin.ManageKMSTrust',
         'TrustedAdmin.ManageTrustedHosts',
         'TrustedAdmin.ReadAttestingSSO',
         'TrustedAdmin.ReadKMSTrust',
         'TrustedAdmin.ReadStsInfo',
         'TrustedAdmin.ReadTrustedHosts',
         'TrustedAdmin.RetrieveHostMetadata',
         'TrustedAdmin.RetrieveTPMHostCertificates'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = -6,
      system = true,
      name = 'NoCryptoAdmin',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'NoCryptoAdmin',
         summary = 'NoCryptoAdmin'
      },
      privilege = (str) [
         'Alarm.Acknowledge',
         'Alarm.Create',
         'Alarm.Delete',
         'Alarm.DisableActions',
         'Alarm.Edit',
         'Alarm.SetStatus',
         'Alarm.ToggleEnableOnEntity',
         'Authorization.ModifyPermissions',
         'Authorization.ModifyPrivileges',
         'Authorization.ModifyRoles',
         'Authorization.ReassignRolePermissions',
         'AutoDeploy.Host.AssociateMachine',
         'AutoDeploy.Profile.Create',
         'AutoDeploy.Profile.Edit',
         'AutoDeploy.Rule.Create',
         'AutoDeploy.Rule.Delete',
         'AutoDeploy.Rule.Edit',
         'AutoDeploy.RuleSet.Activate',
         'AutoDeploy.RuleSet.Edit',
         'Certificate.Manage',
         'CertificateAuthority.Administer',
         'CertificateAuthority.Manage',
         'CertificateManagement.Administer',
         'CertificateManagement.Manage',
         'Cns.Searchable',
         'ComputePolicy.Manage',
         'ContentLibrary.AddLibraryItem',
         'ContentLibrary.AddSubscription',
         'ContentLibrary.CheckInTemplate',
         'ContentLibrary.CheckOutTemplate',
         'ContentLibrary.CreateLocalLibrary',
         'ContentLibrary.CreateSubscribedLibrary',
         'ContentLibrary.DeleteLibraryItem',
         'ContentLibrary.DeleteLocalLibrary',
         'ContentLibrary.DeleteSubscribedLibrary',
         'ContentLibrary.DeleteSubscription',
         'ContentLibrary.DownloadSession',
         'ContentLibrary.EvictLibraryItem',
         'ContentLibrary.EvictSubscribedLibrary',
         'ContentLibrary.GetConfiguration',
         'ContentLibrary.ImportStorage',
         'ContentLibrary.ManageClusterRegistryResource',
         'ContentLibrary.ManageRegistry',
         'ContentLibrary.ManageRegistryProject',
         'ContentLibrary.ProbeSubscription',
         'ContentLibrary.PublishLibrary',
         'ContentLibrary.PublishLibraryItem',
         'ContentLibrary.ReadStorage',
         'ContentLibrary.SyncLibrary',
         'ContentLibrary.SyncLibraryItem',
         'ContentLibrary.TypeIntrospection',
         'ContentLibrary.UpdateConfiguration',
         'ContentLibrary.UpdateLibrary',
         'ContentLibrary.UpdateLibraryItem',
         'ContentLibrary.UpdateLocalLibrary',
         'ContentLibrary.UpdateSession',
         'ContentLibrary.UpdateSubscribedLibrary',
         'ContentLibrary.UpdateSubscription',
         'DVPortgroup.Create',
         'DVPortgroup.Delete',
         'DVPortgroup.Ipfix',
         'DVPortgroup.Modify',
         'DVPortgroup.PolicyOp',
         'DVPortgroup.ScopeOp',
         'DVSwitch.Create',
         'DVSwitch.Delete',
         'DVSwitch.HostOp',
         'DVSwitch.Ipfix',
         'DVSwitch.Modify',
         'DVSwitch.Move',
         'DVSwitch.PolicyOp',
         'DVSwitch.PortConfig',
         'DVSwitch.PortSetting',
         'DVSwitch.ResourceManagement',
         'DVSwitch.Vspan',
         'Datacenter.Create',
         'Datacenter.Delete',
         'Datacenter.IpPoolConfig',
         'Datacenter.IpPoolQueryAllocations',
         'Datacenter.IpPoolReleaseIp',
         'Datacenter.Move',
         'Datacenter.Reconfigure',
         'Datacenter.Rename',
         'Datastore.AllocateSpace',
         'Datastore.Browse',
         'Datastore.Config',
         'Datastore.Delete',
         'Datastore.DeleteFile',
         'Datastore.FileManagement',
         'Datastore.Move',
         'Datastore.Rename',
         'Datastore.UpdateVirtualMachineFiles',
         'Datastore.UpdateVirtualMachineMetadata',
         'EAM.Config',
         'EAM.Modify',
         'EAM.View',
         'Extension.Register',
         'Extension.Unregister',
         'Extension.Update',
         'ExternalStatsProvider.Register',
         'ExternalStatsProvider.Unregister',
         'ExternalStatsProvider.Update',
         'Folder.Create',
         'Folder.Delete',
         'Folder.Move',
         'Folder.Rename',
         'Global.CancelTask',
         'Global.CapacityPlanning',
         'Global.DisableMethods',
         'Global.EnableMethods',
         'Global.GlobalTag',
         'Global.Health',
         'Global.Licenses',
         'Global.LogEvent',
         'Global.ManageCustomFields',
         'Global.Proxy',
         'Global.ScriptAction',
         'Global.ServiceManagers',
         'Global.SetCustomField',
         'Global.Settings',
         'Global.SystemTag',
         'Global.VCServer',
         'HLM.Manage',
         'HealthUpdateProvider.Register',
         'HealthUpdateProvider.Unregister',
         'HealthUpdateProvider.Update',
         'Host.Cim.CimInteraction',
         'Host.Config.AdvancedConfig',
         'Host.Config.AuthenticationStore',
         'Host.Config.AutoStart',
         'Host.Config.Connection',
         'Host.Config.DateTime',
         'Host.Config.Firmware',
         'Host.Config.GuestStore',
         'Host.Config.HyperThreading',
         'Host.Config.Image',
         'Host.Config.Maintenance',
         'Host.Config.Memory',
         'Host.Config.NetService',
         'Host.Config.Network',
         'Host.Config.Nvdimm',
         'Host.Config.Patch',
         'Host.Config.PciPassthru',
         'Host.Config.Power',
         'Host.Config.ProductLocker',
         'Host.Config.Quarantine',
         'Host.Config.Resources',
         'Host.Config.Settings',
         'Host.Config.Snmp',
         'Host.Config.Storage',
         'Host.Config.SystemManagement',
         'Host.Hbr.HbrManagement',
         'Host.Inventory.CreateCluster',
         'Host.Inventory.DeleteCluster',
         'Host.Inventory.EditCluster',
         'Host.Inventory.ManageClusterLifecyle',
         'Host.Inventory.MoveCluster',
         'Host.Inventory.MoveHost',
         'Host.Inventory.RemoveHostFromCluster',
         'Host.Inventory.RenameCluster',
         'Host.Local.CreateVM',
         'Host.Local.DeleteVM',
         'Host.Local.InstallAgent',
         'Host.Local.ReconfigVM',
         'Infraprofile.Read',
         'Infraprofile.Write',
         'IntercomNamespace.Read',
         'IntercomNamespace.Write',
         'InventoryService.Tagging.AttachTag',
         'InventoryService.Tagging.CreateCategory',
         'InventoryService.Tagging.CreateTag',
         'InventoryService.Tagging.DeleteCategory',
         'InventoryService.Tagging.DeleteTag',
         'InventoryService.Tagging.EditCategory',
         'InventoryService.Tagging.EditTag',
         'InventoryService.Tagging.ModifyUsedByForCategory',
         'InventoryService.Tagging.ModifyUsedByForTag',
         'InventoryService.Tagging.ObjectAttachable',
         'NIMBLESTORAGE.Datacenter.com.nimblestorage.hi.View',
         'NIMBLESTORAGE.Datastore.Snapshot.com.nimblestorage.hi.Create',
         'NIMBLESTORAGE.Datastore.Snapshot.com.nimblestorage.hi.Delete',
         'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.Clone',
         'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.Create',
         'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.Delete',
         'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.Edit',
         'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.Grow',
         'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.View',
         'NIMBLESTORAGE.NimbleGroup.ManageGroup',
         'NIMBLESTORAGE.NimbleGroup.groupId.406793085223850253',
         'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.Claim',
         'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.ConfigureVss',
         'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.Delete',
         'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.Purge',
         'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.Restore',
         'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.Undelete',
         'NIMBLESTORAGE.VirtualMachine.View.com.nimblestorage.hi.Deleted',
         'NIMBLESTORAGE.VirtualMachine.View.com.nimblestorage.hi.Local',
         'NIMBLESTORAGE.VirtualMachine.View.com.nimblestorage.hi.Replicated',
         'Namespaces.Configure',
         'Namespaces.Manage',
         'Network.Assign',
         'Network.Config',
         'Network.Delete',
         'Network.Move',
         'Nsx.Manage',
         'Nsx.ModifyAll',
         'Nsx.Read',
         'Observability.Admin',
         'Performance.ModifyIntervals',
         'Plugin.Management',
         'Profile.Clear',
         'Profile.Create',
         'Profile.Delete',
         'Profile.Edit',
         'Profile.Export',
         'Profile.View',
         'Resource.ApplyRecommendation',
         'Resource.AssignVAppToPool',
         'Resource.AssignVMToPool',
         'Resource.ColdMigrate',
         'Resource.CreatePool',
         'Resource.DeletePool',
         'Resource.EditPool',
         'Resource.HotMigrate',
         'Resource.MovePool',
         'Resource.QueryVMotion',
         'Resource.RenamePool',
         'ScheduledTask.Create',
         'ScheduledTask.Delete',
         'ScheduledTask.Edit',
         'ScheduledTask.Run',
         'ServiceAccount.Administer',
         'ServiceAccount.ManageAccount',
         'ServiceAccount.ManagePassword',
         'Sessions.GlobalMessage',
         'Sessions.ImpersonateUser',
         'Sessions.TerminateSession',
         'Sessions.ValidateSession',
         'SettingsStore.Manage',
         'StoragePod.Config',
         'StorageProfile.Update',
         'StorageProfile.View',
         'StorageViews.ConfigureService',
         'StorageViews.View',
         'System.Anonymous',
         'System.Read',
         'System.View',
         'Task.Create',
         'Task.Update',
         'TenantManager.Query',
         'TenantManager.Update',
         'TransferService.Manage',
         'TransferService.Monitor',
         'Trust.Administer',
         'Trust.Manage',
         'VApp.ApplicationConfig',
         'VApp.AssignResourcePool',
         'VApp.AssignVApp',
         'VApp.AssignVM',
         'VApp.Clone',
         'VApp.Create',
         'VApp.Delete',
         'VApp.Export',
         'VApp.ExtractOvfEnvironment',
         'VApp.Import',
         'VApp.InstanceConfig',
         'VApp.ManagedByConfig',
         'VApp.Move',
         'VApp.PowerOff',
         'VApp.PowerOn',
         'VApp.PullFromUrls',
         'VApp.Rename',
         'VApp.ResourceConfig',
         'VApp.Suspend',
         'VApp.Unregister',
         'VcIdentityProviders.Create',
         'VcIdentityProviders.Manage',
         'VcIdentityProviders.Read',
         'VcIntegrity.Baseline.com.vmware.vcIntegrity.AssignBaselines',
         'VcIntegrity.Baseline.com.vmware.vcIntegrity.ManageBaselines',
         'VcIntegrity.FileUpload.com.vmware.vcIntegrity.ImportFile',
         'VcIntegrity.General.com.vmware.vcIntegrity.Configure',
         'VcIntegrity.HardwareCompatibility.Read',
         'VcIntegrity.Updates.com.vmware.vcIntegrity.Remediate',
         'VcIntegrity.Updates.com.vmware.vcIntegrity.Scan',
         'VcIntegrity.Updates.com.vmware.vcIntegrity.Stage',
         'VcIntegrity.Updates.com.vmware.vcIntegrity.ViewStatus',
         'VcIntegrity.lifecycleGeneral.Read',
         'VcIntegrity.lifecycleGeneral.Write',
         'VcIntegrity.lifecycleHealth.Read',
         'VcIntegrity.lifecycleHealth.Write',
         'VcIntegrity.lifecycleSettings.Read',
         'VcIntegrity.lifecycleSettings.Write',
         'VcIntegrity.lifecycleSoftwareRemediation.Read',
         'VcIntegrity.lifecycleSoftwareRemediation.Write',
         'VcIntegrity.lifecycleSoftwareSpecification.Read',
         'VcIntegrity.lifecycleSoftwareSpecification.Write',
         'VcLifecycle.View',
         'VirtualMachine.Config.AddExistingDisk',
         'VirtualMachine.Config.AddNewDisk',
         'VirtualMachine.Config.AddRemoveDevice',
         'VirtualMachine.Config.AdvancedConfig',
         'VirtualMachine.Config.Annotation',
         'VirtualMachine.Config.CPUCount',
         'VirtualMachine.Config.ChangeTracking',
         'VirtualMachine.Config.DiskExtend',
         'VirtualMachine.Config.DiskLease',
         'VirtualMachine.Config.EditDevice',
         'VirtualMachine.Config.HostUSBDevice',
         'VirtualMachine.Config.ManagedBy',
         'VirtualMachine.Config.Memory',
         'VirtualMachine.Config.MksControl',
         'VirtualMachine.Config.QueryFTCompatibility',
         'VirtualMachine.Config.QueryUnownedFiles',
         'VirtualMachine.Config.RawDevice',
         'VirtualMachine.Config.ReloadFromPath',
         'VirtualMachine.Config.RemoveDisk',
         'VirtualMachine.Config.Rename',
         'VirtualMachine.Config.ResetGuestInfo',
         'VirtualMachine.Config.Resource',
         'VirtualMachine.Config.Settings',
         'VirtualMachine.Config.SwapPlacement',
         'VirtualMachine.Config.ToggleForkParent',
         'VirtualMachine.Config.UpgradeVirtualHardware',
         'VirtualMachine.GuestOperations.Execute',
         'VirtualMachine.GuestOperations.Modify',
         'VirtualMachine.GuestOperations.ModifyAliases',
         'VirtualMachine.GuestOperations.Query',
         'VirtualMachine.GuestOperations.QueryAliases',
         'VirtualMachine.Hbr.ConfigureReplication',
         'VirtualMachine.Hbr.MonitorReplication',
         'VirtualMachine.Hbr.ReplicaManagement',
         'VirtualMachine.Interact.AnswerQuestion',
         'VirtualMachine.Interact.Backup',
         'VirtualMachine.Interact.ConsoleInteract',
         'VirtualMachine.Interact.CreateScreenshot',
         'VirtualMachine.Interact.CreateSecondary',
         'VirtualMachine.Interact.DefragmentAllDisks',
         'VirtualMachine.Interact.DeviceConnection',
         'VirtualMachine.Interact.DisableSecondary',
         'VirtualMachine.Interact.DnD',
         'VirtualMachine.Interact.EnableSecondary',
         'VirtualMachine.Interact.MakePrimary',
         'VirtualMachine.Interact.Pause',
         'VirtualMachine.Interact.PowerOff',
         'VirtualMachine.Interact.PowerOn',
         'VirtualMachine.Interact.PutUsbScanCodes',
         'VirtualMachine.Interact.Record',
         'VirtualMachine.Interact.Replay',
         'VirtualMachine.Interact.Reset',
         'VirtualMachine.Interact.SESparseMaintenance',
         'VirtualMachine.Interact.SetCDMedia',
         'VirtualMachine.Interact.SetFloppyMedia',
         'VirtualMachine.Interact.Suspend',
         'VirtualMachine.Interact.SuspendToMemory',
         'VirtualMachine.Interact.TerminateFaultTolerantVM',
         'VirtualMachine.Interact.ToolsInstall',
         'VirtualMachine.Interact.TurnOffFaultTolerance',
         'VirtualMachine.Inventory.Create',
         'VirtualMachine.Inventory.CreateFromExisting',
         'VirtualMachine.Inventory.Delete',
         'VirtualMachine.Inventory.Move',
         'VirtualMachine.Inventory.Register',
         'VirtualMachine.Inventory.Unregister',
         'VirtualMachine.Namespace.Event',
         'VirtualMachine.Namespace.EventNotify',
         'VirtualMachine.Namespace.Management',
         'VirtualMachine.Namespace.ModifyContent',
         'VirtualMachine.Namespace.Query',
         'VirtualMachine.Namespace.ReadContent',
         'VirtualMachine.Provisioning.Clone',
         'VirtualMachine.Provisioning.CloneTemplate',
         'VirtualMachine.Provisioning.CreateTemplateFromVM',
         'VirtualMachine.Provisioning.Customize',
         'VirtualMachine.Provisioning.DeployTemplate',
         'VirtualMachine.Provisioning.DiskRandomAccess',
         'VirtualMachine.Provisioning.DiskRandomRead',
         'VirtualMachine.Provisioning.FileRandomAccess',
         'VirtualMachine.Provisioning.GetVmFiles',
         'VirtualMachine.Provisioning.MarkAsTemplate',
         'VirtualMachine.Provisioning.MarkAsVM',
         'VirtualMachine.Provisioning.ModifyCustSpecs',
         'VirtualMachine.Provisioning.PromoteDisks',
         'VirtualMachine.Provisioning.PutVmFiles',
         'VirtualMachine.Provisioning.ReadCustSpecs',
         'VirtualMachine.State.CreateSnapshot',
         'VirtualMachine.State.RemoveSnapshot',
         'VirtualMachine.State.RenameSnapshot',
         'VirtualMachine.State.RevertToSnapshot',
         'Vsan.Cluster.ShallowRekey',
         'vService.CreateDependency',
         'vService.DestroyDependency',
         'vService.ReconfigureDependency',
         'vService.UpdateDependency',
         'vStats.CollectAny',
         'vStats.QueryAny',
         'vStats.Settings'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = -5,
      system = true,
      name = 'NoAccess',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'NoAccess',
         summary = 'NoAccess'
      },
      privilege = (str) []
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = -4,
      system = true,
      name = 'Anonymous',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'Anonymous',
         summary = 'Anonymous'
      },
      privilege = (str) [
         'System.Anonymous'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = -3,
      system = true,
      name = 'View',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'View',
         summary = 'View'
      },
      privilege = (str) [
         'System.Anonymous',
         'System.View'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = -2,
      system = true,
      name = 'ReadOnly',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'ReadOnly',
         summary = 'ReadOnly'
      },
      privilege = (str) [
         'System.Anonymous',
         'System.Read',
         'System.View'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = -1,
      system = true,
      name = 'Admin',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'Admin',
         summary = 'Admin'
      },
      privilege = (str) [
         'Alarm.Acknowledge',
         'Alarm.Create',
         'Alarm.Delete',
         'Alarm.DisableActions',
         'Alarm.Edit',
         'Alarm.SetStatus',
         'Alarm.ToggleEnableOnEntity',
         'Authorization.ModifyPermissions',
         'Authorization.ModifyPrivileges',
         'Authorization.ModifyRoles',
         'Authorization.ReassignRolePermissions',
         'AutoDeploy.Host.AssociateMachine',
         'AutoDeploy.Profile.Create',
         'AutoDeploy.Profile.Edit',
         'AutoDeploy.Rule.Create',
         'AutoDeploy.Rule.Delete',
         'AutoDeploy.Rule.Edit',
         'AutoDeploy.RuleSet.Activate',
         'AutoDeploy.RuleSet.Edit',
         'Certificate.Manage',
         'CertificateAuthority.Administer',
         'CertificateAuthority.Manage',
         'CertificateManagement.Administer',
         'CertificateManagement.Manage',
         'Cns.Searchable',
         'ComputePolicy.Manage',
         'ContentLibrary.AddLibraryItem',
         'ContentLibrary.AddSubscription',
         'ContentLibrary.CheckInTemplate',
         'ContentLibrary.CheckOutTemplate',
         'ContentLibrary.CreateLocalLibrary',
         'ContentLibrary.CreateSubscribedLibrary',
         'ContentLibrary.DeleteLibraryItem',
         'ContentLibrary.DeleteLocalLibrary',
         'ContentLibrary.DeleteSubscribedLibrary',
         'ContentLibrary.DeleteSubscription',
         'ContentLibrary.DownloadSession',
         'ContentLibrary.EvictLibraryItem',
         'ContentLibrary.EvictSubscribedLibrary',
         'ContentLibrary.GetConfiguration',
         'ContentLibrary.ImportStorage',
         'ContentLibrary.ManageClusterRegistryResource',
         'ContentLibrary.ManageRegistry',
         'ContentLibrary.ManageRegistryProject',
         'ContentLibrary.ProbeSubscription',
         'ContentLibrary.PublishLibrary',
         'ContentLibrary.PublishLibraryItem',
         'ContentLibrary.ReadStorage',
         'ContentLibrary.SyncLibrary',
         'ContentLibrary.SyncLibraryItem',
         'ContentLibrary.TypeIntrospection',
         'ContentLibrary.UpdateConfiguration',
         'ContentLibrary.UpdateLibrary',
         'ContentLibrary.UpdateLibraryItem',
         'ContentLibrary.UpdateLocalLibrary',
         'ContentLibrary.UpdateSession',
         'ContentLibrary.UpdateSubscribedLibrary',
         'ContentLibrary.UpdateSubscription',
         'Cryptographer.Access',
         'Cryptographer.AddDisk',
         'Cryptographer.Clone',
         'Cryptographer.Decrypt',
         'Cryptographer.Encrypt',
         'Cryptographer.EncryptNew',
         'Cryptographer.ManageEncryptionPolicy',
         'Cryptographer.ManageKeyServers',
         'Cryptographer.ManageKeys',
         'Cryptographer.Migrate',
         'Cryptographer.Recrypt',
         'Cryptographer.RegisterHost',
         'Cryptographer.RegisterVM',
         'DVPortgroup.Create',
         'DVPortgroup.Delete',
         'DVPortgroup.Ipfix',
         'DVPortgroup.Modify',
         'DVPortgroup.PolicyOp',
         'DVPortgroup.ScopeOp',
         'DVSwitch.Create',
         'DVSwitch.Delete',
         'DVSwitch.HostOp',
         'DVSwitch.Ipfix',
         'DVSwitch.Modify',
         'DVSwitch.Move',
         'DVSwitch.PolicyOp',
         'DVSwitch.PortConfig',
         'DVSwitch.PortSetting',
         'DVSwitch.ResourceManagement',
         'DVSwitch.Vspan',
         'Datacenter.Create',
         'Datacenter.Delete',
         'Datacenter.IpPoolConfig',
         'Datacenter.IpPoolQueryAllocations',
         'Datacenter.IpPoolReleaseIp',
         'Datacenter.Move',
         'Datacenter.Reconfigure',
         'Datacenter.Rename',
         'Datastore.AllocateSpace',
         'Datastore.Browse',
         'Datastore.Config',
         'Datastore.Delete',
         'Datastore.DeleteFile',
         'Datastore.FileManagement',
         'Datastore.Move',
         'Datastore.Rename',
         'Datastore.UpdateVirtualMachineFiles',
         'Datastore.UpdateVirtualMachineMetadata',
         'EAM.Config',
         'EAM.Modify',
         'EAM.View',
         'Extension.Register',
         'Extension.Unregister',
         'Extension.Update',
         'ExternalStatsProvider.Register',
         'ExternalStatsProvider.Unregister',
         'ExternalStatsProvider.Update',
         'Folder.Create',
         'Folder.Delete',
         'Folder.Move',
         'Folder.Rename',
         'Global.CancelTask',
         'Global.CapacityPlanning',
         'Global.Diagnostics',
         'Global.DisableMethods',
         'Global.EnableMethods',
         'Global.GlobalTag',
         'Global.Health',
         'Global.Licenses',
         'Global.LogEvent',
         'Global.ManageCustomFields',
         'Global.Proxy',
         'Global.ScriptAction',
         'Global.ServiceManagers',
         'Global.SetCustomField',
         'Global.Settings',
         'Global.SystemTag',
         'Global.VCServer',
         'HLM.Manage',
         'HealthUpdateProvider.Register',
         'HealthUpdateProvider.Unregister',
         'HealthUpdateProvider.Update',
         'Host.Cim.CimInteraction',
         'Host.Config.AdvancedConfig',
         'Host.Config.AuthenticationStore',
         'Host.Config.AutoStart',
         'Host.Config.Connection',
         'Host.Config.DateTime',
         'Host.Config.Firmware',
         'Host.Config.GuestStore',
         'Host.Config.HyperThreading',
         'Host.Config.Image',
         'Host.Config.Maintenance',
         'Host.Config.Memory',
         'Host.Config.NetService',
         'Host.Config.Network',
         'Host.Config.Nvdimm',
         'Host.Config.Patch',
         'Host.Config.PciPassthru',
         'Host.Config.Power',
         'Host.Config.ProductLocker',
         'Host.Config.Quarantine',
         'Host.Config.Resources',
         'Host.Config.Settings',
         'Host.Config.Snmp',
         'Host.Config.Storage',
         'Host.Config.SystemManagement',
         'Host.Hbr.HbrManagement',
         'Host.Inventory.AddHostToCluster',
         'Host.Inventory.AddStandaloneHost',
         'Host.Inventory.CreateCluster',
         'Host.Inventory.DeleteCluster',
         'Host.Inventory.EditCluster',
         'Host.Inventory.ManageClusterLifecyle',
         'Host.Inventory.MoveCluster',
         'Host.Inventory.MoveHost',
         'Host.Inventory.RemoveHostFromCluster',
         'Host.Inventory.RenameCluster',
         'Host.Local.CreateVM',
         'Host.Local.DeleteVM',
         'Host.Local.InstallAgent',
         'Host.Local.ManageUserGroups',
         'Host.Local.ReconfigVM',
         'Infraprofile.Read',
         'Infraprofile.Write',
         'IntercomNamespace.Read',
         'IntercomNamespace.Write',
         'InventoryService.Tagging.AttachTag',
         'InventoryService.Tagging.CreateCategory',
         'InventoryService.Tagging.CreateTag',
         'InventoryService.Tagging.DeleteCategory',
         'InventoryService.Tagging.DeleteTag',
         'InventoryService.Tagging.EditCategory',
         'InventoryService.Tagging.EditTag',
         'InventoryService.Tagging.ModifyUsedByForCategory',
         'InventoryService.Tagging.ModifyUsedByForTag',
         'InventoryService.Tagging.ObjectAttachable',
         'NIMBLESTORAGE.Datacenter.com.nimblestorage.hi.View',
         'NIMBLESTORAGE.Datastore.Snapshot.com.nimblestorage.hi.Create',
         'NIMBLESTORAGE.Datastore.Snapshot.com.nimblestorage.hi.Delete',
         'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.Clone',
         'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.Create',
         'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.Delete',
         'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.Edit',
         'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.Grow',
         'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.View',
         'NIMBLESTORAGE.NimbleGroup.ManageGroup',
         'NIMBLESTORAGE.NimbleGroup.groupId.406793085223850253',
         'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.Claim',
         'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.ConfigureVss',
         'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.Delete',
         'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.Purge',
         'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.Restore',
         'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.Undelete',
         'NIMBLESTORAGE.VirtualMachine.View.com.nimblestorage.hi.Deleted',
         'NIMBLESTORAGE.VirtualMachine.View.com.nimblestorage.hi.Local',
         'NIMBLESTORAGE.VirtualMachine.View.com.nimblestorage.hi.Replicated',
         'Namespaces.Configure',
         'Namespaces.Manage',
         'Network.Assign',
         'Network.Config',
         'Network.Delete',
         'Network.Move',
         'Nsx.Manage',
         'Nsx.ModifyAll',
         'Nsx.Read',
         'Observability.Admin',
         'Performance.ModifyIntervals',
         'Plugin.Management',
         'Profile.Clear',
         'Profile.Create',
         'Profile.Delete',
         'Profile.Edit',
         'Profile.Export',
         'Profile.View',
         'Resource.ApplyRecommendation',
         'Resource.AssignVAppToPool',
         'Resource.AssignVMToPool',
         'Resource.ColdMigrate',
         'Resource.CreatePool',
         'Resource.DeletePool',
         'Resource.EditPool',
         'Resource.HotMigrate',
         'Resource.MovePool',
         'Resource.QueryVMotion',
         'Resource.RenamePool',
         'ScheduledTask.Create',
         'ScheduledTask.Delete',
         'ScheduledTask.Edit',
         'ScheduledTask.Run',
         'ServiceAccount.Administer',
         'ServiceAccount.ManageAccount',
         'ServiceAccount.ManagePassword',
         'Sessions.GlobalMessage',
         'Sessions.ImpersonateUser',
         'Sessions.TerminateSession',
         'Sessions.ValidateSession',
         'SettingsStore.Manage',
         'StoragePod.Config',
         'StorageProfile.Update',
         'StorageProfile.View',
         'StorageViews.ConfigureService',
         'StorageViews.View',
         'System.Anonymous',
         'System.Read',
         'System.View',
         'Task.Create',
         'Task.Update',
         'TenantManager.Query',
         'TenantManager.Update',
         'TransferService.Manage',
         'TransferService.Monitor',
         'Trust.Administer',
         'Trust.Manage',
         'TrustedAdmin.ConfigureHostCertificates',
         'TrustedAdmin.ConfigureHostMetadata',
         'TrustedAdmin.ConfigureTokenConversionPolicy',
         'TrustedAdmin.ManageAttestingSSO',
         'TrustedAdmin.ManageKMSTrust',
         'TrustedAdmin.ManageTrustedHosts',
         'TrustedAdmin.ReadAttestingSSO',
         'TrustedAdmin.ReadKMSTrust',
         'TrustedAdmin.ReadStsInfo',
         'TrustedAdmin.ReadTrustedHosts',
         'TrustedAdmin.RetrieveHostMetadata',
         'TrustedAdmin.RetrieveTPMHostCertificates',
         'VApp.ApplicationConfig',
         'VApp.AssignResourcePool',
         'VApp.AssignVApp',
         'VApp.AssignVM',
         'VApp.Clone',
         'VApp.Create',
         'VApp.Delete',
         'VApp.Export',
         'VApp.ExtractOvfEnvironment',
         'VApp.Import',
         'VApp.InstanceConfig',
         'VApp.ManagedByConfig',
         'VApp.Move',
         'VApp.PowerOff',
         'VApp.PowerOn',
         'VApp.PullFromUrls',
         'VApp.Rename',
         'VApp.ResourceConfig',
         'VApp.Suspend',
         'VApp.Unregister',
         'VcIdentityProviders.Create',
         'VcIdentityProviders.Manage',
         'VcIdentityProviders.Read',
         'VcIntegrity.Baseline.com.vmware.vcIntegrity.AssignBaselines',
         'VcIntegrity.Baseline.com.vmware.vcIntegrity.ManageBaselines',
         'VcIntegrity.FileUpload.com.vmware.vcIntegrity.ImportFile',
         'VcIntegrity.General.com.vmware.vcIntegrity.Configure',
         'VcIntegrity.HardwareCompatibility.Read',
         'VcIntegrity.Updates.com.vmware.vcIntegrity.Remediate',
         'VcIntegrity.Updates.com.vmware.vcIntegrity.Scan',
         'VcIntegrity.Updates.com.vmware.vcIntegrity.Stage',
         'VcIntegrity.Updates.com.vmware.vcIntegrity.ViewStatus',
         'VcIntegrity.lifecycleGeneral.Read',
         'VcIntegrity.lifecycleGeneral.Write',
         'VcIntegrity.lifecycleHealth.Read',
         'VcIntegrity.lifecycleHealth.Write',
         'VcIntegrity.lifecycleSettings.Read',
         'VcIntegrity.lifecycleSettings.Write',
         'VcIntegrity.lifecycleSoftwareRemediation.Read',
         'VcIntegrity.lifecycleSoftwareRemediation.Write',
         'VcIntegrity.lifecycleSoftwareSpecification.Read',
         'VcIntegrity.lifecycleSoftwareSpecification.Write',
         'VcLifecycle.View',
         'VirtualMachine.Config.AddExistingDisk',
         'VirtualMachine.Config.AddNewDisk',
         'VirtualMachine.Config.AddRemoveDevice',
         'VirtualMachine.Config.AdvancedConfig',
         'VirtualMachine.Config.Annotation',
         'VirtualMachine.Config.CPUCount',
         'VirtualMachine.Config.ChangeTracking',
         'VirtualMachine.Config.DiskExtend',
         'VirtualMachine.Config.DiskLease',
         'VirtualMachine.Config.EditDevice',
         'VirtualMachine.Config.HostUSBDevice',
         'VirtualMachine.Config.ManagedBy',
         'VirtualMachine.Config.Memory',
         'VirtualMachine.Config.MksControl',
         'VirtualMachine.Config.QueryFTCompatibility',
         'VirtualMachine.Config.QueryUnownedFiles',
         'VirtualMachine.Config.RawDevice',
         'VirtualMachine.Config.ReloadFromPath',
         'VirtualMachine.Config.RemoveDisk',
         'VirtualMachine.Config.Rename',
         'VirtualMachine.Config.ResetGuestInfo',
         'VirtualMachine.Config.Resource',
         'VirtualMachine.Config.Settings',
         'VirtualMachine.Config.SwapPlacement',
         'VirtualMachine.Config.ToggleForkParent',
         'VirtualMachine.Config.UpgradeVirtualHardware',
         'VirtualMachine.GuestOperations.Execute',
         'VirtualMachine.GuestOperations.Modify',
         'VirtualMachine.GuestOperations.ModifyAliases',
         'VirtualMachine.GuestOperations.Query',
         'VirtualMachine.GuestOperations.QueryAliases',
         'VirtualMachine.Hbr.ConfigureReplication',
         'VirtualMachine.Hbr.MonitorReplication',
         'VirtualMachine.Hbr.ReplicaManagement',
         'VirtualMachine.Interact.AnswerQuestion',
         'VirtualMachine.Interact.Backup',
         'VirtualMachine.Interact.ConsoleInteract',
         'VirtualMachine.Interact.CreateScreenshot',
         'VirtualMachine.Interact.CreateSecondary',
         'VirtualMachine.Interact.DefragmentAllDisks',
         'VirtualMachine.Interact.DeviceConnection',
         'VirtualMachine.Interact.DisableSecondary',
         'VirtualMachine.Interact.DnD',
         'VirtualMachine.Interact.EnableSecondary',
         'VirtualMachine.Interact.GuestControl',
         'VirtualMachine.Interact.MakePrimary',
         'VirtualMachine.Interact.Pause',
         'VirtualMachine.Interact.PowerOff',
         'VirtualMachine.Interact.PowerOn',
         'VirtualMachine.Interact.PutUsbScanCodes',
         'VirtualMachine.Interact.Record',
         'VirtualMachine.Interact.Replay',
         'VirtualMachine.Interact.Reset',
         'VirtualMachine.Interact.SESparseMaintenance',
         'VirtualMachine.Interact.SetCDMedia',
         'VirtualMachine.Interact.SetFloppyMedia',
         'VirtualMachine.Interact.Suspend',
         'VirtualMachine.Interact.SuspendToMemory',
         'VirtualMachine.Interact.TerminateFaultTolerantVM',
         'VirtualMachine.Interact.ToolsInstall',
         'VirtualMachine.Interact.TurnOffFaultTolerance',
         'VirtualMachine.Inventory.Create',
         'VirtualMachine.Inventory.CreateFromExisting',
         'VirtualMachine.Inventory.Delete',
         'VirtualMachine.Inventory.Move',
         'VirtualMachine.Inventory.Register',
         'VirtualMachine.Inventory.Unregister',
         'VirtualMachine.Namespace.Event',
         'VirtualMachine.Namespace.EventNotify',
         'VirtualMachine.Namespace.Management',
         'VirtualMachine.Namespace.ModifyContent',
         'VirtualMachine.Namespace.Query',
         'VirtualMachine.Namespace.ReadContent',
         'VirtualMachine.Provisioning.Clone',
         'VirtualMachine.Provisioning.CloneTemplate',
         'VirtualMachine.Provisioning.CreateTemplateFromVM',
         'VirtualMachine.Provisioning.Customize',
         'VirtualMachine.Provisioning.DeployTemplate',
         'VirtualMachine.Provisioning.DiskRandomAccess',
         'VirtualMachine.Provisioning.DiskRandomRead',
         'VirtualMachine.Provisioning.FileRandomAccess',
         'VirtualMachine.Provisioning.GetVmFiles',
         'VirtualMachine.Provisioning.MarkAsTemplate',
         'VirtualMachine.Provisioning.MarkAsVM',
         'VirtualMachine.Provisioning.ModifyCustSpecs',
         'VirtualMachine.Provisioning.PromoteDisks',
         'VirtualMachine.Provisioning.PutVmFiles',
         'VirtualMachine.Provisioning.ReadCustSpecs',
         'VirtualMachine.State.CreateSnapshot',
         'VirtualMachine.State.RemoveSnapshot',
         'VirtualMachine.State.RenameSnapshot',
         'VirtualMachine.State.RevertToSnapshot',
         'Vsan.Cluster.ShallowRekey',
         'vService.CreateDependency',
         'vService.DestroyDependency',
         'vService.ReconfigureDependency',
         'vService.UpdateDependency',
         'vStats.CollectAny',
         'vStats.QueryAny',
         'vStats.Settings'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = 4,
      system = false,
      name = 'VirtualMachinePowerUser',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'VirtualMachinePowerUser',
         summary = 'VirtualMachinePowerUser'
      },
      privilege = (str) [
         'Datastore.Browse',
         'Global.CancelTask',
         'ScheduledTask.Create',
         'ScheduledTask.Delete',
         'ScheduledTask.Edit',
         'ScheduledTask.Run',
         'System.Anonymous',
         'System.Read',
         'System.View',
         'VirtualMachine.Config.AddExistingDisk',
         'VirtualMachine.Config.AddNewDisk',
         'VirtualMachine.Config.AddRemoveDevice',
         'VirtualMachine.Config.AdvancedConfig',
         'VirtualMachine.Config.CPUCount',
         'VirtualMachine.Config.DiskLease',
         'VirtualMachine.Config.EditDevice',
         'VirtualMachine.Config.Memory',
         'VirtualMachine.Config.RemoveDisk',
         'VirtualMachine.Config.Rename',
         'VirtualMachine.Config.ResetGuestInfo',
         'VirtualMachine.Config.Resource',
         'VirtualMachine.Config.Settings',
         'VirtualMachine.Config.UpgradeVirtualHardware',
         'VirtualMachine.Interact.AnswerQuestion',
         'VirtualMachine.Interact.ConsoleInteract',
         'VirtualMachine.Interact.DeviceConnection',
         'VirtualMachine.Interact.GuestControl',
         'VirtualMachine.Interact.PowerOff',
         'VirtualMachine.Interact.PowerOn',
         'VirtualMachine.Interact.Reset',
         'VirtualMachine.Interact.SetCDMedia',
         'VirtualMachine.Interact.SetFloppyMedia',
         'VirtualMachine.Interact.Suspend',
         'VirtualMachine.Interact.ToolsInstall',
         'VirtualMachine.State.CreateSnapshot',
         'VirtualMachine.State.RemoveSnapshot',
         'VirtualMachine.State.RenameSnapshot',
         'VirtualMachine.State.RevertToSnapshot'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = 5,
      system = false,
      name = 'VirtualMachineUser',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'VirtualMachineUser',
         summary = 'VirtualMachineUser'
      },
      privilege = (str) [
         'Global.CancelTask',
         'ScheduledTask.Create',
         'ScheduledTask.Delete',
         'ScheduledTask.Edit',
         'ScheduledTask.Run',
         'System.Anonymous',
         'System.Read',
         'System.View',
         'VirtualMachine.Interact.AnswerQuestion',
         'VirtualMachine.Interact.ConsoleInteract',
         'VirtualMachine.Interact.DeviceConnection',
         'VirtualMachine.Interact.PowerOff',
         'VirtualMachine.Interact.PowerOn',
         'VirtualMachine.Interact.Reset',
         'VirtualMachine.Interact.SetCDMedia',
         'VirtualMachine.Interact.SetFloppyMedia',
         'VirtualMachine.Interact.Suspend',
         'VirtualMachine.Interact.ToolsInstall'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = 6,
      system = false,
      name = 'ResourcePoolAdministrator',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'ResourcePoolAdministrator',
         summary = 'ResourcePoolAdministrator'
      },
      privilege = (str) [
         'Alarm.Create',
         'Alarm.Delete',
         'Alarm.Edit',
         'Authorization.ModifyPermissions',
         'Datastore.Browse',
         'Folder.Create',
         'Folder.Delete',
         'Folder.Move',
         'Folder.Rename',
         'Global.CancelTask',
         'Global.LogEvent',
         'Global.SetCustomField',
         'Resource.AssignVMToPool',
         'Resource.ColdMigrate',
         'Resource.CreatePool',
         'Resource.DeletePool',
         'Resource.EditPool',
         'Resource.HotMigrate',
         'Resource.MovePool',
         'Resource.QueryVMotion',
         'Resource.RenamePool',
         'ScheduledTask.Create',
         'ScheduledTask.Delete',
         'ScheduledTask.Edit',
         'ScheduledTask.Run',
         'System.Anonymous',
         'System.Read',
         'System.View',
         'VirtualMachine.Config.AddExistingDisk',
         'VirtualMachine.Config.AddNewDisk',
         'VirtualMachine.Config.AddRemoveDevice',
         'VirtualMachine.Config.AdvancedConfig',
         'VirtualMachine.Config.CPUCount',
         'VirtualMachine.Config.DiskLease',
         'VirtualMachine.Config.EditDevice',
         'VirtualMachine.Config.Memory',
         'VirtualMachine.Config.RawDevice',
         'VirtualMachine.Config.RemoveDisk',
         'VirtualMachine.Config.Rename',
         'VirtualMachine.Config.ResetGuestInfo',
         'VirtualMachine.Config.Resource',
         'VirtualMachine.Config.Settings',
         'VirtualMachine.Config.UpgradeVirtualHardware',
         'VirtualMachine.Interact.AnswerQuestion',
         'VirtualMachine.Interact.ConsoleInteract',
         'VirtualMachine.Interact.DeviceConnection',
         'VirtualMachine.Interact.PowerOff',
         'VirtualMachine.Interact.PowerOn',
         'VirtualMachine.Interact.Reset',
         'VirtualMachine.Interact.SetCDMedia',
         'VirtualMachine.Interact.SetFloppyMedia',
         'VirtualMachine.Interact.Suspend',
         'VirtualMachine.Interact.ToolsInstall',
         'VirtualMachine.Inventory.Create',
         'VirtualMachine.Inventory.CreateFromExisting',
         'VirtualMachine.Inventory.Delete',
         'VirtualMachine.Inventory.Move',
         'VirtualMachine.Inventory.Register',
         'VirtualMachine.Inventory.Unregister',
         'VirtualMachine.Provisioning.Clone',
         'VirtualMachine.Provisioning.CloneTemplate',
         'VirtualMachine.Provisioning.CreateTemplateFromVM',
         'VirtualMachine.Provisioning.Customize',
         'VirtualMachine.Provisioning.DeployTemplate',
         'VirtualMachine.Provisioning.DiskRandomAccess',
         'VirtualMachine.Provisioning.DiskRandomRead',
         'VirtualMachine.Provisioning.GetVmFiles',
         'VirtualMachine.Provisioning.MarkAsTemplate',
         'VirtualMachine.Provisioning.MarkAsVM',
         'VirtualMachine.Provisioning.ModifyCustSpecs',
         'VirtualMachine.Provisioning.PutVmFiles',
         'VirtualMachine.Provisioning.ReadCustSpecs',
         'VirtualMachine.State.CreateSnapshot',
         'VirtualMachine.State.RemoveSnapshot',
         'VirtualMachine.State.RenameSnapshot',
         'VirtualMachine.State.RevertToSnapshot'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = 7,
      system = false,
      name = 'VMwareConsolidatedBackupUser',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'VMwareConsolidatedBackupUser',
         summary = 'VMwareConsolidatedBackupUser'
      },
      privilege = (str) [
         'System.Anonymous',
         'System.Read',
         'System.View',
         'VirtualMachine.Config.DiskLease',
         'VirtualMachine.Provisioning.DiskRandomRead',
         'VirtualMachine.Provisioning.GetVmFiles',
         'VirtualMachine.State.CreateSnapshot',
         'VirtualMachine.State.RemoveSnapshot'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = 8,
      system = false,
      name = 'DatastoreConsumer',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'DatastoreConsumer',
         summary = 'DatastoreConsumer'
      },
      privilege = (str) [
         'Datastore.AllocateSpace',
         'System.Anonymous',
         'System.Read',
         'System.View'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = 9,
      system = false,
      name = 'NetworkConsumer',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'NetworkConsumer',
         summary = 'NetworkConsumer'
      },
      privilege = (str) [
         'Network.Assign',
         'System.Anonymous',
         'System.Read',
         'System.View'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = 10,
      system = false,
      name = 'VirtualMachineConsoleUser',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'VirtualMachineConsoleUser',
         summary = 'VirtualMachineConsoleUser'
      },
      privilege = (str) [
         'System.Anonymous',
         'System.Read',
         'System.View',
         'VirtualMachine.Interact.AnswerQuestion',
         'VirtualMachine.Interact.ConsoleInteract',
         'VirtualMachine.Interact.DeviceConnection',
         'VirtualMachine.Interact.PowerOff',
         'VirtualMachine.Interact.PowerOn',
         'VirtualMachine.Interact.Reset',
         'VirtualMachine.Interact.SetCDMedia',
         'VirtualMachine.Interact.SetFloppyMedia',
         'VirtualMachine.Interact.Suspend',
         'VirtualMachine.Interact.ToolsInstall'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = 11,
      system = false,
      name = 'AutoUpdateUser',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'AutoUpdateUser',
         summary = 'AutoUpdateUser'
      },
      privilege = (str) [
         'Global.LogEvent',
         'System.Anonymous',
         'System.Read',
         'System.View'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = 1001,
      system = false,
      name = 'InventoryService.Tagging.TaggingAdmin',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'InventoryService.Tagging.TaggingAdmin',
         summary = 'InventoryService.Tagging.TaggingAdmin'
      },
      privilege = (str) [
         'InventoryService.Tagging.AttachTag',
         'InventoryService.Tagging.CreateCategory',
         'InventoryService.Tagging.CreateTag',
         'InventoryService.Tagging.DeleteCategory',
         'InventoryService.Tagging.DeleteTag',
         'InventoryService.Tagging.EditCategory',
         'InventoryService.Tagging.EditTag',
         'InventoryService.Tagging.ModifyUsedByForCategory',
         'InventoryService.Tagging.ModifyUsedByForTag',
         'InventoryService.Tagging.ObjectAttachable',
         'System.Anonymous',
         'System.Read',
         'System.View'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = 1002,
      system = false,
      name = 'SyncUsers',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'SyncUsers',
         summary = 'SyncUsers'
      },
      privilege = (str) [
         'CertificateManagement.Manage',
         'HLM.Manage',
         'IntercomNamespace.Read',
         'IntercomNamespace.Write',
         'InventoryService.Tagging.AttachTag',
         'InventoryService.Tagging.CreateCategory',
         'InventoryService.Tagging.CreateTag',
         'InventoryService.Tagging.DeleteCategory',
         'InventoryService.Tagging.DeleteTag',
         'InventoryService.Tagging.EditCategory',
         'InventoryService.Tagging.EditTag',
         'InventoryService.Tagging.ModifyUsedByForCategory',
         'InventoryService.Tagging.ModifyUsedByForTag',
         'SettingsStore.Manage',
         'System.Anonymous',
         'System.Read',
         'System.View',
         'Trust.Manage'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = 1003,
      system = false,
      name = 'vSphere Client Solution User',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'vSphere Client Solution User',
         summary = 'vSphere Client Solution User'
      },
      privilege = (str) [
         'IntercomNamespace.Read',
         'System.Anonymous',
         'System.Read',
         'System.View',
         'Task.Create',
         'Task.Update',
         'VcIdentityProviders.Read'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = 1004,
      system = false,
      name = 'WorkloadStorageManagement',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'WorkloadStorageManagement',
         summary = 'WorkloadStorageManagement'
      },
      privilege = (str) [
         'Cns.Searchable',
         'Datastore.AllocateSpace',
         'Datastore.FileManagement',
         'EAM.Modify',
         'Resource.AssignVMToPool',
         'StorageProfile.View',
         'System.Anonymous',
         'System.Read',
         'System.View',
         'VirtualMachine.Config.AddExistingDisk',
         'VirtualMachine.Config.AddNewDisk',
         'VirtualMachine.Config.AddRemoveDevice',
         'VirtualMachine.Config.RemoveDisk',
         'VirtualMachine.Config.Settings',
         'VirtualMachine.Inventory.Create',
         'VirtualMachine.Inventory.Delete'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = 1005,
      system = false,
      name = 'vSphereKubernetesManager',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'vSphereKubernetesManager',
         summary = 'vSphereKubernetesManager'
      },
      privilege = (str) [
         'ContentLibrary.ManageClusterRegistryResource',
         'Datastore.AllocateSpace',
         'Resource.AssignVMToPool',
         'System.Anonymous',
         'System.Read',
         'System.View',
         'VirtualMachine.Config.AddNewDisk',
         'VirtualMachine.Config.AddRemoveDevice',
         'VirtualMachine.Interact.PowerOff',
         'VirtualMachine.Interact.PowerOn',
         'VirtualMachine.Inventory.Create',
         'VirtualMachine.Inventory.Delete'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = 1006,
      system = false,
      name = 'com.vmware.Content.Registry.Admin',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'com.vmware.Content.Registry.Admin',
         summary = 'com.vmware.Content.Registry.Admin'
      },
      privilege = (str) [
         'ContentLibrary.ManageRegistry',
         'ContentLibrary.ManageRegistryProject',
         'System.Anonymous',
         'System.Read',
         'System.View'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = 1007,
      system = false,
      name = 'SupervisorServiceCluster',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'SupervisorServiceCluster',
         summary = 'SupervisorServiceCluster'
      },
      privilege = (str) [
         'Host.Inventory.EditCluster',
         'Namespaces.Configure',
         'System.Anonymous',
         'System.Read',
         'System.View'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = 1008,
      system = false,
      name = 'SupervisorServiceRootFolder',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'SupervisorServiceRootFolder',
         summary = 'SupervisorServiceRootFolder'
      },
      privilege = (str) [
         'StorageProfile.Update',
         'StorageProfile.View',
         'System.Anonymous',
         'System.Read',
         'System.View'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = 1009,
      system = false,
      name = 'SupervisorServiceGlobal',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'SupervisorServiceGlobal',
         summary = 'SupervisorServiceGlobal'
      },
      privilege = (str) [
         'CertificateAuthority.Manage',
         'System.Anonymous',
         'System.Read',
         'System.View'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = 1010,
      system = false,
      name = 'AppdApplianceUser',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'AppdApplianceUser',
         summary = 'AppdApplianceUser'
      },
      privilege = (str) [
         'Alarm.Acknowledge',
         'Alarm.Create',
         'Alarm.Delete',
         'Alarm.DisableActions',
         'Alarm.Edit',
         'Alarm.SetStatus',
         'Alarm.ToggleEnableOnEntity',
         'Extension.Register',
         'Extension.Unregister',
         'Extension.Update',
         'Global.CancelTask',
         'Global.Licenses',
         'Global.Settings',
         'Host.Config.AdvancedConfig',
         'Host.Config.Patch',
         'Sessions.ImpersonateUser',
         'Sessions.TerminateSession',
         'System.Anonymous',
         'System.Read',
         'System.View',
         'VirtualMachine.Config.AdvancedConfig',
         'VirtualMachine.Config.Settings',
         'VirtualMachine.Interact.PowerOff',
         'VirtualMachine.Interact.PowerOn',
         'VirtualMachine.Interact.Reset',
         'VirtualMachine.Interact.Suspend',
         'VirtualMachine.Interact.ToolsInstall',
         'VirtualMachine.State.CreateSnapshot',
         'VirtualMachine.State.RemoveSnapshot',
         'VirtualMachine.State.RevertToSnapshot'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = 1021,
      system = false,
      name = 'VMOperatorController',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'VMOperatorController',
         summary = 'VMOperatorController'
      },
      privilege = (str) [
         'Datastore.AllocateSpace',
         'Folder.Create',
         'Folder.Delete',
         'InventoryService.Tagging.AttachTag',
         'InventoryService.Tagging.ObjectAttachable',
         'Network.Assign',
         'Network.Config',
         'Network.Delete',
         'Network.Move',
         'Resource.ApplyRecommendation',
         'Resource.AssignVAppToPool',
         'Resource.AssignVMToPool',
         'Resource.ColdMigrate',
         'Resource.CreatePool',
         'Resource.DeletePool',
         'Resource.EditPool',
         'Resource.HotMigrate',
         'Resource.MovePool',
         'Resource.QueryVMotion',
         'Resource.RenamePool',
         'System.Anonymous',
         'System.Read',
         'System.View',
         'Task.Create',
         'Task.Update',
         'VApp.ApplicationConfig',
         'VApp.AssignResourcePool',
         'VApp.AssignVApp',
         'VApp.AssignVM',
         'VApp.Clone',
         'VApp.Create',
         'VApp.Delete',
         'VApp.Export',
         'VApp.ExtractOvfEnvironment',
         'VApp.Import',
         'VApp.InstanceConfig',
         'VApp.ManagedByConfig',
         'VApp.Move',
         'VApp.PowerOff',
         'VApp.PowerOn',
         'VApp.PullFromUrls',
         'VApp.Rename',
         'VApp.ResourceConfig',
         'VApp.Suspend',
         'VApp.Unregister',
         'VirtualMachine.Config.AddExistingDisk',
         'VirtualMachine.Config.AddNewDisk',
         'VirtualMachine.Config.AddRemoveDevice',
         'VirtualMachine.Config.AdvancedConfig',
         'VirtualMachine.Config.Annotation',
         'VirtualMachine.Config.CPUCount',
         'VirtualMachine.Config.ChangeTracking',
         'VirtualMachine.Config.DiskExtend',
         'VirtualMachine.Config.DiskLease',
         'VirtualMachine.Config.EditDevice',
         'VirtualMachine.Config.HostUSBDevice',
         'VirtualMachine.Config.ManagedBy',
         'VirtualMachine.Config.Memory',
         'VirtualMachine.Config.MksControl',
         'VirtualMachine.Config.QueryFTCompatibility',
         'VirtualMachine.Config.QueryUnownedFiles',
         'VirtualMachine.Config.RawDevice',
         'VirtualMachine.Config.ReloadFromPath',
         'VirtualMachine.Config.RemoveDisk',
         'VirtualMachine.Config.Rename',
         'VirtualMachine.Config.ResetGuestInfo',
         'VirtualMachine.Config.Resource',
         'VirtualMachine.Config.Settings',
         'VirtualMachine.Config.SwapPlacement',
         'VirtualMachine.Config.ToggleForkParent',
         'VirtualMachine.Config.UpgradeVirtualHardware',
         'VirtualMachine.GuestOperations.Execute',
         'VirtualMachine.GuestOperations.Modify',
         'VirtualMachine.GuestOperations.ModifyAliases',
         'VirtualMachine.GuestOperations.Query',
         'VirtualMachine.GuestOperations.QueryAliases',
         'VirtualMachine.Hbr.ConfigureReplication',
         'VirtualMachine.Hbr.MonitorReplication',
         'VirtualMachine.Hbr.ReplicaManagement',
         'VirtualMachine.Interact.AnswerQuestion',
         'VirtualMachine.Interact.Backup',
         'VirtualMachine.Interact.ConsoleInteract',
         'VirtualMachine.Interact.CreateScreenshot',
         'VirtualMachine.Interact.CreateSecondary',
         'VirtualMachine.Interact.DefragmentAllDisks',
         'VirtualMachine.Interact.DeviceConnection',
         'VirtualMachine.Interact.DisableSecondary',
         'VirtualMachine.Interact.DnD',
         'VirtualMachine.Interact.EnableSecondary',
         'VirtualMachine.Interact.GuestControl',
         'VirtualMachine.Interact.MakePrimary',
         'VirtualMachine.Interact.Pause',
         'VirtualMachine.Interact.PowerOff',
         'VirtualMachine.Interact.PowerOn',
         'VirtualMachine.Interact.PutUsbScanCodes',
         'VirtualMachine.Interact.Record',
         'VirtualMachine.Interact.Replay',
         'VirtualMachine.Interact.Reset',
         'VirtualMachine.Interact.SESparseMaintenance',
         'VirtualMachine.Interact.SetCDMedia',
         'VirtualMachine.Interact.SetFloppyMedia',
         'VirtualMachine.Interact.Suspend',
         'VirtualMachine.Interact.SuspendToMemory',
         'VirtualMachine.Interact.TerminateFaultTolerantVM',
         'VirtualMachine.Interact.ToolsInstall',
         'VirtualMachine.Interact.TurnOffFaultTolerance',
         'VirtualMachine.Inventory.Create',
         'VirtualMachine.Inventory.CreateFromExisting',
         'VirtualMachine.Inventory.Delete',
         'VirtualMachine.Inventory.Move',
         'VirtualMachine.Inventory.Register',
         'VirtualMachine.Inventory.Unregister',
         'VirtualMachine.Namespace.Event',
         'VirtualMachine.Namespace.EventNotify',
         'VirtualMachine.Namespace.Management',
         'VirtualMachine.Namespace.ModifyContent',
         'VirtualMachine.Namespace.Query',
         'VirtualMachine.Namespace.ReadContent',
         'VirtualMachine.Provisioning.Clone',
         'VirtualMachine.Provisioning.CloneTemplate',
         'VirtualMachine.Provisioning.CreateTemplateFromVM',
         'VirtualMachine.Provisioning.Customize',
         'VirtualMachine.Provisioning.DeployTemplate',
         'VirtualMachine.Provisioning.DiskRandomAccess',
         'VirtualMachine.Provisioning.DiskRandomRead',
         'VirtualMachine.Provisioning.FileRandomAccess',
         'VirtualMachine.Provisioning.GetVmFiles',
         'VirtualMachine.Provisioning.MarkAsTemplate',
         'VirtualMachine.Provisioning.MarkAsVM',
         'VirtualMachine.Provisioning.ModifyCustSpecs',
         'VirtualMachine.Provisioning.PromoteDisks',
         'VirtualMachine.Provisioning.PutVmFiles',
         'VirtualMachine.Provisioning.ReadCustSpecs',
         'VirtualMachine.State.CreateSnapshot',
         'VirtualMachine.State.RemoveSnapshot',
         'VirtualMachine.State.RenameSnapshot',
         'VirtualMachine.State.RevertToSnapshot'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = 1022,
      system = false,
      name = 'VMOperatorControllerGlobal',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'VMOperatorControllerGlobal',
         summary = 'VMOperatorControllerGlobal'
      },
      privilege = (str) [
         'ContentLibrary.CheckOutTemplate',
         'ContentLibrary.DownloadSession',
         'ContentLibrary.SyncLibrary',
         'ContentLibrary.SyncLibraryItem',
         'InventoryService.Tagging.AttachTag',
         'InventoryService.Tagging.ObjectAttachable',
         'System.Anonymous',
         'System.Read',
         'System.View'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = 490548812,
      system = false,
      name = 'NsxViAdministrator',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'NsxViAdministrator',
         summary = 'NsxViAdministrator'
      },
      privilege = (str) [
         'Nsx.Manage',
         'System.Anonymous',
         'System.Read',
         'System.View'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = 525514943,
      system = false,
      name = 'vStatsUser',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'vStatsUser',
         summary = 'vStatsUser'
      },
      privilege = (str) [
         'System.Anonymous',
         'System.Read',
         'System.View',
         'vStats.CollectAny',
         'vStats.QueryAny'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = 621425699,
      system = false,
      name = 'admin2 Role',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'admin2 Role',
         summary = 'admin2 Role'
      },
      privilege = (str) [
         'Alarm.Acknowledge',
         'Alarm.Create',
         'Alarm.Delete',
         'Alarm.DisableActions',
         'Alarm.Edit',
         'Alarm.SetStatus',
         'Alarm.ToggleEnableOnEntity',
         'Authorization.ModifyPermissions',
         'Authorization.ModifyPrivileges',
         'Authorization.ModifyRoles',
         'Authorization.ReassignRolePermissions',
         'AutoDeploy.Host.AssociateMachine',
         'AutoDeploy.Profile.Create',
         'AutoDeploy.Profile.Edit',
         'AutoDeploy.Rule.Create',
         'AutoDeploy.Rule.Delete',
         'AutoDeploy.Rule.Edit',
         'AutoDeploy.RuleSet.Activate',
         'AutoDeploy.RuleSet.Edit',
         'Certificate.Manage',
         'CertificateAuthority.Administer',
         'CertificateAuthority.Manage',
         'CertificateManagement.Administer',
         'CertificateManagement.Manage',
         'Cns.Searchable',
         'ComputePolicy.Manage',
         'ContentLibrary.AddLibraryItem',
         'ContentLibrary.AddSubscription',
         'ContentLibrary.CheckInTemplate',
         'ContentLibrary.CheckOutTemplate',
         'ContentLibrary.CreateLocalLibrary',
         'ContentLibrary.CreateSubscribedLibrary',
         'ContentLibrary.DeleteLibraryItem',
         'ContentLibrary.DeleteLocalLibrary',
         'ContentLibrary.DeleteSubscribedLibrary',
         'ContentLibrary.DeleteSubscription',
         'ContentLibrary.DownloadSession',
         'ContentLibrary.EvictLibraryItem',
         'ContentLibrary.EvictSubscribedLibrary',
         'ContentLibrary.GetConfiguration',
         'ContentLibrary.ImportStorage',
         'ContentLibrary.ManageClusterRegistryResource',
         'ContentLibrary.ManageRegistry',
         'ContentLibrary.ManageRegistryProject',
         'ContentLibrary.ProbeSubscription',
         'ContentLibrary.PublishLibrary',
         'ContentLibrary.PublishLibraryItem',
         'ContentLibrary.ReadStorage',
         'ContentLibrary.SyncLibrary',
         'ContentLibrary.SyncLibraryItem',
         'ContentLibrary.TypeIntrospection',
         'ContentLibrary.UpdateConfiguration',
         'ContentLibrary.UpdateLibrary',
         'ContentLibrary.UpdateLibraryItem',
         'ContentLibrary.UpdateLocalLibrary',
         'ContentLibrary.UpdateSession',
         'ContentLibrary.UpdateSubscribedLibrary',
         'ContentLibrary.UpdateSubscription',
         'Cryptographer.Access',
         'Cryptographer.AddDisk',
         'Cryptographer.Clone',
         'Cryptographer.Decrypt',
         'Cryptographer.Encrypt',
         'Cryptographer.EncryptNew',
         'Cryptographer.ManageEncryptionPolicy',
         'Cryptographer.ManageKeyServers',
         'Cryptographer.ManageKeys',
         'Cryptographer.Migrate',
         'Cryptographer.Recrypt',
         'Cryptographer.RegisterHost',
         'Cryptographer.RegisterVM',
         'DVPortgroup.Create',
         'DVPortgroup.Delete',
         'DVPortgroup.Ipfix',
         'DVPortgroup.Modify',
         'DVPortgroup.PolicyOp',
         'DVPortgroup.ScopeOp',
         'DVSwitch.Create',
         'DVSwitch.Delete',
         'DVSwitch.HostOp',
         'DVSwitch.Ipfix',
         'DVSwitch.Modify',
         'DVSwitch.Move',
         'DVSwitch.PolicyOp',
         'DVSwitch.PortConfig',
         'DVSwitch.PortSetting',
         'DVSwitch.ResourceManagement',
         'DVSwitch.Vspan',
         'Datacenter.Create',
         'Datacenter.Delete',
         'Datacenter.IpPoolConfig',
         'Datacenter.IpPoolQueryAllocations',
         'Datacenter.IpPoolReleaseIp',
         'Datacenter.Move',
         'Datacenter.Reconfigure',
         'Datacenter.Rename',
         'Datastore.AllocateSpace',
         'Datastore.Browse',
         'Datastore.Config',
         'Datastore.Delete',
         'Datastore.DeleteFile',
         'Datastore.FileManagement',
         'Datastore.Move',
         'Datastore.Rename',
         'Datastore.UpdateVirtualMachineFiles',
         'Datastore.UpdateVirtualMachineMetadata',
         'EAM.Config',
         'EAM.Modify',
         'EAM.View',
         'Extension.Register',
         'Extension.Unregister',
         'Extension.Update',
         'ExternalStatsProvider.Register',
         'ExternalStatsProvider.Unregister',
         'ExternalStatsProvider.Update',
         'Folder.Create',
         'Folder.Delete',
         'Folder.Move',
         'Folder.Rename',
         'Global.CancelTask',
         'Global.CapacityPlanning',
         'Global.Diagnostics',
         'Global.DisableMethods',
         'Global.EnableMethods',
         'Global.GlobalTag',
         'Global.Health',
         'Global.Licenses',
         'Global.LogEvent',
         'Global.ManageCustomFields',
         'Global.Proxy',
         'Global.ScriptAction',
         'Global.ServiceManagers',
         'Global.SetCustomField',
         'Global.Settings',
         'Global.SystemTag',
         'Global.VCServer',
         'HLM.Manage',
         'HealthUpdateProvider.Register',
         'HealthUpdateProvider.Unregister',
         'HealthUpdateProvider.Update',
         'Host.Cim.CimInteraction',
         'Host.Config.AdvancedConfig',
         'Host.Config.AuthenticationStore',
         'Host.Config.AutoStart',
         'Host.Config.Connection',
         'Host.Config.DateTime',
         'Host.Config.Firmware',
         'Host.Config.GuestStore',
         'Host.Config.HyperThreading',
         'Host.Config.Image',
         'Host.Config.Maintenance',
         'Host.Config.Memory',
         'Host.Config.NetService',
         'Host.Config.Network',
         'Host.Config.Nvdimm',
         'Host.Config.Patch',
         'Host.Config.PciPassthru',
         'Host.Config.Power',
         'Host.Config.ProductLocker',
         'Host.Config.Quarantine',
         'Host.Config.Resources',
         'Host.Config.Settings',
         'Host.Config.Snmp',
         'Host.Config.Storage',
         'Host.Config.SystemManagement',
         'Host.Hbr.HbrManagement',
         'Host.Inventory.AddHostToCluster',
         'Host.Inventory.AddStandaloneHost',
         'Host.Inventory.CreateCluster',
         'Host.Inventory.DeleteCluster',
         'Host.Inventory.EditCluster',
         'Host.Inventory.ManageClusterLifecyle',
         'Host.Inventory.MoveCluster',
         'Host.Inventory.MoveHost',
         'Host.Inventory.RemoveHostFromCluster',
         'Host.Inventory.RenameCluster',
         'Host.Local.CreateVM',
         'Host.Local.DeleteVM',
         'Host.Local.InstallAgent',
         'Host.Local.ManageUserGroups',
         'Host.Local.ReconfigVM',
         'Infraprofile.Read',
         'Infraprofile.Write',
         'IntercomNamespace.Read',
         'IntercomNamespace.Write',
         'InventoryService.Tagging.AttachTag',
         'InventoryService.Tagging.CreateCategory',
         'InventoryService.Tagging.CreateTag',
         'InventoryService.Tagging.DeleteCategory',
         'InventoryService.Tagging.DeleteTag',
         'InventoryService.Tagging.EditCategory',
         'InventoryService.Tagging.EditTag',
         'InventoryService.Tagging.ModifyUsedByForCategory',
         'InventoryService.Tagging.ModifyUsedByForTag',
         'InventoryService.Tagging.ObjectAttachable',
         'NIMBLESTORAGE.Datacenter.com.nimblestorage.hi.View',
         'NIMBLESTORAGE.Datastore.Snapshot.com.nimblestorage.hi.Create',
         'NIMBLESTORAGE.Datastore.Snapshot.com.nimblestorage.hi.Delete',
         'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.Clone',
         'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.Create',
         'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.Delete',
         'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.Edit',
         'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.Grow',
         'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.View',
         'NIMBLESTORAGE.NimbleGroup.ManageGroup',
         'NIMBLESTORAGE.NimbleGroup.groupId.406793085223850253',
         'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.Claim',
         'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.ConfigureVss',
         'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.Delete',
         'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.Purge',
         'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.Restore',
         'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.Undelete',
         'NIMBLESTORAGE.VirtualMachine.View.com.nimblestorage.hi.Deleted',
         'NIMBLESTORAGE.VirtualMachine.View.com.nimblestorage.hi.Local',
         'NIMBLESTORAGE.VirtualMachine.View.com.nimblestorage.hi.Replicated',
         'Namespaces.Configure',
         'Namespaces.Manage',
         'Network.Assign',
         'Network.Config',
         'Network.Delete',
         'Network.Move',
         'Nsx.Manage',
         'Nsx.ModifyAll',
         'Nsx.Read',
         'Observability.Admin',
         'Performance.ModifyIntervals',
         'Plugin.Management',
         'Profile.Clear',
         'Profile.Create',
         'Profile.Delete',
         'Profile.Edit',
         'Profile.Export',
         'Profile.View',
         'Resource.ApplyRecommendation',
         'Resource.AssignVAppToPool',
         'Resource.AssignVMToPool',
         'Resource.ColdMigrate',
         'Resource.CreatePool',
         'Resource.DeletePool',
         'Resource.EditPool',
         'Resource.HotMigrate',
         'Resource.MovePool',
         'Resource.QueryVMotion',
         'Resource.RenamePool',
         'ScheduledTask.Create',
         'ScheduledTask.Delete',
         'ScheduledTask.Edit',
         'ScheduledTask.Run',
         'ServiceAccount.Administer',
         'ServiceAccount.ManageAccount',
         'ServiceAccount.ManagePassword',
         'Sessions.GlobalMessage',
         'Sessions.ImpersonateUser',
         'Sessions.TerminateSession',
         'Sessions.ValidateSession',
         'SettingsStore.Manage',
         'StoragePod.Config',
         'StorageProfile.Update',
         'StorageProfile.View',
         'StorageViews.ConfigureService',
         'StorageViews.View',
         'System.Anonymous',
         'System.Read',
         'System.View',
         'Task.Create',
         'Task.Update',
         'TenantManager.Query',
         'TenantManager.Update',
         'TransferService.Manage',
         'TransferService.Monitor',
         'Trust.Administer',
         'Trust.Manage',
         'VApp.ApplicationConfig',
         'VApp.AssignResourcePool',
         'VApp.AssignVApp',
         'VApp.AssignVM',
         'VApp.Clone',
         'VApp.Create',
         'VApp.Delete',
         'VApp.Export',
         'VApp.ExtractOvfEnvironment',
         'VApp.Import',
         'VApp.InstanceConfig',
         'VApp.ManagedByConfig',
         'VApp.Move',
         'VApp.PowerOff',
         'VApp.PowerOn',
         'VApp.PullFromUrls',
         'VApp.Rename',
         'VApp.ResourceConfig',
         'VApp.Suspend',
         'VApp.Unregister',
         'VcIdentityProviders.Create',
         'VcIdentityProviders.Manage',
         'VcIdentityProviders.Read',
         'VcIntegrity.Baseline.com.vmware.vcIntegrity.AssignBaselines',
         'VcIntegrity.Baseline.com.vmware.vcIntegrity.ManageBaselines',
         'VcIntegrity.FileUpload.com.vmware.vcIntegrity.ImportFile',
         'VcIntegrity.General.com.vmware.vcIntegrity.Configure',
         'VcIntegrity.HardwareCompatibility.Read',
         'VcIntegrity.Updates.com.vmware.vcIntegrity.Remediate',
         'VcIntegrity.Updates.com.vmware.vcIntegrity.Scan',
         'VcIntegrity.Updates.com.vmware.vcIntegrity.Stage',
         'VcIntegrity.Updates.com.vmware.vcIntegrity.ViewStatus',
         'VcIntegrity.lifecycleGeneral.Read',
         'VcIntegrity.lifecycleGeneral.Write',
         'VcIntegrity.lifecycleHealth.Read',
         'VcIntegrity.lifecycleHealth.Write',
         'VcIntegrity.lifecycleSettings.Read',
         'VcIntegrity.lifecycleSettings.Write',
         'VcIntegrity.lifecycleSoftwareRemediation.Read',
         'VcIntegrity.lifecycleSoftwareRemediation.Write',
         'VcIntegrity.lifecycleSoftwareSpecification.Read',
         'VcIntegrity.lifecycleSoftwareSpecification.Write',
         'VcLifecycle.View',
         'VirtualMachine.Config.AddExistingDisk',
         'VirtualMachine.Config.AddNewDisk',
         'VirtualMachine.Config.AddRemoveDevice',
         'VirtualMachine.Config.AdvancedConfig',
         'VirtualMachine.Config.Annotation',
         'VirtualMachine.Config.CPUCount',
         'VirtualMachine.Config.ChangeTracking',
         'VirtualMachine.Config.DiskExtend',
         'VirtualMachine.Config.EditDevice',
         'VirtualMachine.Config.HostUSBDevice',
         'VirtualMachine.Config.ManagedBy',
         'VirtualMachine.Config.Memory',
         'VirtualMachine.Config.MksControl',
         'VirtualMachine.Config.QueryFTCompatibility',
         'VirtualMachine.Config.QueryUnownedFiles',
         'VirtualMachine.Config.RawDevice',
         'VirtualMachine.Config.ReloadFromPath',
         'VirtualMachine.Config.RemoveDisk',
         'VirtualMachine.Config.Rename',
         'VirtualMachine.Config.ResetGuestInfo',
         'VirtualMachine.Config.Resource',
         'VirtualMachine.Config.Settings',
         'VirtualMachine.Config.SwapPlacement',
         'VirtualMachine.Config.ToggleForkParent',
         'VirtualMachine.Config.UpgradeVirtualHardware',
         'VirtualMachine.GuestOperations.Execute',
         'VirtualMachine.GuestOperations.Modify',
         'VirtualMachine.GuestOperations.ModifyAliases',
         'VirtualMachine.GuestOperations.Query',
         'VirtualMachine.GuestOperations.QueryAliases',
         'VirtualMachine.Hbr.ConfigureReplication',
         'VirtualMachine.Hbr.MonitorReplication',
         'VirtualMachine.Hbr.ReplicaManagement',
         'VirtualMachine.Interact.AnswerQuestion',
         'VirtualMachine.Interact.Backup',
         'VirtualMachine.Interact.ConsoleInteract',
         'VirtualMachine.Interact.CreateScreenshot',
         'VirtualMachine.Interact.CreateSecondary',
         'VirtualMachine.Interact.DefragmentAllDisks',
         'VirtualMachine.Interact.DeviceConnection',
         'VirtualMachine.Interact.DisableSecondary',
         'VirtualMachine.Interact.DnD',
         'VirtualMachine.Interact.EnableSecondary',
         'VirtualMachine.Interact.GuestControl',
         'VirtualMachine.Interact.MakePrimary',
         'VirtualMachine.Interact.Pause',
         'VirtualMachine.Interact.PowerOff',
         'VirtualMachine.Interact.PowerOn',
         'VirtualMachine.Interact.PutUsbScanCodes',
         'VirtualMachine.Interact.Record',
         'VirtualMachine.Interact.Replay',
         'VirtualMachine.Interact.Reset',
         'VirtualMachine.Interact.SESparseMaintenance',
         'VirtualMachine.Interact.SetCDMedia',
         'VirtualMachine.Interact.SetFloppyMedia',
         'VirtualMachine.Interact.Suspend',
         'VirtualMachine.Interact.SuspendToMemory',
         'VirtualMachine.Interact.TerminateFaultTolerantVM',
         'VirtualMachine.Interact.ToolsInstall',
         'VirtualMachine.Interact.TurnOffFaultTolerance',
         'VirtualMachine.Inventory.Create',
         'VirtualMachine.Inventory.CreateFromExisting',
         'VirtualMachine.Inventory.Delete',
         'VirtualMachine.Inventory.Move',
         'VirtualMachine.Inventory.Register',
         'VirtualMachine.Inventory.Unregister',
         'VirtualMachine.Namespace.Event',
         'VirtualMachine.Namespace.EventNotify',
         'VirtualMachine.Namespace.Management',
         'VirtualMachine.Namespace.ModifyContent',
         'VirtualMachine.Namespace.Query',
         'VirtualMachine.Namespace.ReadContent',
         'VirtualMachine.Provisioning.Clone',
         'VirtualMachine.Provisioning.CloneTemplate',
         'VirtualMachine.Provisioning.CreateTemplateFromVM',
         'VirtualMachine.Provisioning.Customize',
         'VirtualMachine.Provisioning.DeployTemplate',
         'VirtualMachine.Provisioning.DiskRandomAccess',
         'VirtualMachine.Provisioning.DiskRandomRead',
         'VirtualMachine.Provisioning.FileRandomAccess',
         'VirtualMachine.Provisioning.MarkAsTemplate',
         'VirtualMachine.Provisioning.MarkAsVM',
         'VirtualMachine.Provisioning.ModifyCustSpecs',
         'VirtualMachine.Provisioning.PromoteDisks',
         'VirtualMachine.Provisioning.ReadCustSpecs',
         'VirtualMachine.State.RemoveSnapshot',
         'VirtualMachine.State.RenameSnapshot',
         'VirtualMachine.State.RevertToSnapshot',
         'Vsan.Cluster.ShallowRekey',
         'vService.CreateDependency',
         'vService.DestroyDependency',
         'vService.ReconfigureDependency',
         'vService.UpdateDependency',
         'vStats.CollectAny',
         'vStats.QueryAny',
         'vStats.Settings'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = 1433892634,
      system = false,
      name = 'NsxAuditor',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'NsxAuditor',
         summary = 'NsxAuditor'
      },
      privilege = (str) [
         'Nsx.Read',
         'System.Anonymous',
         'System.Read',
         'System.View'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = 1488418561,
      system = false,
      name = 'NsxAdministrator',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'NsxAdministrator',
         summary = 'NsxAdministrator'
      },
      privilege = (str) [
         'Nsx.ModifyAll',
         'System.Anonymous',
         'System.Read',
         'System.View'
      ]
   },
   (vim.AuthorizationManager.Role) {
      dynamicType = <unset>,
      dynamicProperty = (vmodl.DynamicProperty) [],
      roleId = 1587487975,
      system = false,
      name = 'vStatsAdmin',
      info = (vim.Description) {
         dynamicType = <unset>,
         dynamicProperty = (vmodl.DynamicProperty) [],
         label = 'vStatsAdmin',
         summary = 'vStatsAdmin'
      },
      privilege = (str) [
         'System.Anonymous',
         'System.Read',
         'System.View',
         'vStats.CollectAny',
         'vStats.QueryAny',
         'vStats.Settings'
      ]
   }
]
........user role: 621425699, privileges: (str) [
   'Alarm.Acknowledge',
   'Alarm.Create',
   'Alarm.Delete',
   'Alarm.DisableActions',
   'Alarm.Edit',
   'Alarm.SetStatus',
   'Alarm.ToggleEnableOnEntity',
   'Authorization.ModifyPermissions',
   'Authorization.ModifyPrivileges',
   'Authorization.ModifyRoles',
   'Authorization.ReassignRolePermissions',
   'AutoDeploy.Host.AssociateMachine',
   'AutoDeploy.Profile.Create',
   'AutoDeploy.Profile.Edit',
   'AutoDeploy.Rule.Create',
   'AutoDeploy.Rule.Delete',
   'AutoDeploy.Rule.Edit',
   'AutoDeploy.RuleSet.Activate',
   'AutoDeploy.RuleSet.Edit',
   'Certificate.Manage',
   'CertificateAuthority.Administer',
   'CertificateAuthority.Manage',
   'CertificateManagement.Administer',
   'CertificateManagement.Manage',
   'Cns.Searchable',
   'ComputePolicy.Manage',
   'ContentLibrary.AddLibraryItem',
   'ContentLibrary.AddSubscription',
   'ContentLibrary.CheckInTemplate',
   'ContentLibrary.CheckOutTemplate',
   'ContentLibrary.CreateLocalLibrary',
   'ContentLibrary.CreateSubscribedLibrary',
   'ContentLibrary.DeleteLibraryItem',
   'ContentLibrary.DeleteLocalLibrary',
   'ContentLibrary.DeleteSubscribedLibrary',
   'ContentLibrary.DeleteSubscription',
   'ContentLibrary.DownloadSession',
   'ContentLibrary.EvictLibraryItem',
   'ContentLibrary.EvictSubscribedLibrary',
   'ContentLibrary.GetConfiguration',
   'ContentLibrary.ImportStorage',
   'ContentLibrary.ManageClusterRegistryResource',
   'ContentLibrary.ManageRegistry',
   'ContentLibrary.ManageRegistryProject',
   'ContentLibrary.ProbeSubscription',
   'ContentLibrary.PublishLibrary',
   'ContentLibrary.PublishLibraryItem',
   'ContentLibrary.ReadStorage',
   'ContentLibrary.SyncLibrary',
   'ContentLibrary.SyncLibraryItem',
   'ContentLibrary.TypeIntrospection',
   'ContentLibrary.UpdateConfiguration',
   'ContentLibrary.UpdateLibrary',
   'ContentLibrary.UpdateLibraryItem',
   'ContentLibrary.UpdateLocalLibrary',
   'ContentLibrary.UpdateSession',
   'ContentLibrary.UpdateSubscribedLibrary',
   'ContentLibrary.UpdateSubscription',
   'Cryptographer.Access',
   'Cryptographer.AddDisk',
   'Cryptographer.Clone',
   'Cryptographer.Decrypt',
   'Cryptographer.Encrypt',
   'Cryptographer.EncryptNew',
   'Cryptographer.ManageEncryptionPolicy',
   'Cryptographer.ManageKeyServers',
   'Cryptographer.ManageKeys',
   'Cryptographer.Migrate',
   'Cryptographer.Recrypt',
   'Cryptographer.RegisterHost',
   'Cryptographer.RegisterVM',
   'DVPortgroup.Create',
   'DVPortgroup.Delete',
   'DVPortgroup.Ipfix',
   'DVPortgroup.Modify',
   'DVPortgroup.PolicyOp',
   'DVPortgroup.ScopeOp',
   'DVSwitch.Create',
   'DVSwitch.Delete',
   'DVSwitch.HostOp',
   'DVSwitch.Ipfix',
   'DVSwitch.Modify',
   'DVSwitch.Move',
   'DVSwitch.PolicyOp',
   'DVSwitch.PortConfig',
   'DVSwitch.PortSetting',
   'DVSwitch.ResourceManagement',
   'DVSwitch.Vspan',
   'Datacenter.Create',
   'Datacenter.Delete',
   'Datacenter.IpPoolConfig',
   'Datacenter.IpPoolQueryAllocations',
   'Datacenter.IpPoolReleaseIp',
   'Datacenter.Move',
   'Datacenter.Reconfigure',
   'Datacenter.Rename',
   'Datastore.AllocateSpace',
   'Datastore.Browse',
   'Datastore.Config',
   'Datastore.Delete',
   'Datastore.DeleteFile',
   'Datastore.FileManagement',
   'Datastore.Move',
   'Datastore.Rename',
   'Datastore.UpdateVirtualMachineFiles',
   'Datastore.UpdateVirtualMachineMetadata',
   'EAM.Config',
   'EAM.Modify',
   'EAM.View',
   'Extension.Register',
   'Extension.Unregister',
   'Extension.Update',
   'ExternalStatsProvider.Register',
   'ExternalStatsProvider.Unregister',
   'ExternalStatsProvider.Update',
   'Folder.Create',
   'Folder.Delete',
   'Folder.Move',
   'Folder.Rename',
   'Global.CancelTask',
   'Global.CapacityPlanning',
   'Global.Diagnostics',
   'Global.DisableMethods',
   'Global.EnableMethods',
   'Global.GlobalTag',
   'Global.Health',
   'Global.Licenses',
   'Global.LogEvent',
   'Global.ManageCustomFields',
   'Global.Proxy',
   'Global.ScriptAction',
   'Global.ServiceManagers',
   'Global.SetCustomField',
   'Global.Settings',
   'Global.SystemTag',
   'Global.VCServer',
   'HLM.Manage',
   'HealthUpdateProvider.Register',
   'HealthUpdateProvider.Unregister',
   'HealthUpdateProvider.Update',
   'Host.Cim.CimInteraction',
   'Host.Config.AdvancedConfig',
   'Host.Config.AuthenticationStore',
   'Host.Config.AutoStart',
   'Host.Config.Connection',
   'Host.Config.DateTime',
   'Host.Config.Firmware',
   'Host.Config.GuestStore',
   'Host.Config.HyperThreading',
   'Host.Config.Image',
   'Host.Config.Maintenance',
   'Host.Config.Memory',
   'Host.Config.NetService',
   'Host.Config.Network',
   'Host.Config.Nvdimm',
   'Host.Config.Patch',
   'Host.Config.PciPassthru',
   'Host.Config.Power',
   'Host.Config.ProductLocker',
   'Host.Config.Quarantine',
   'Host.Config.Resources',
   'Host.Config.Settings',
   'Host.Config.Snmp',
   'Host.Config.Storage',
   'Host.Config.SystemManagement',
   'Host.Hbr.HbrManagement',
   'Host.Inventory.AddHostToCluster',
   'Host.Inventory.AddStandaloneHost',
   'Host.Inventory.CreateCluster',
   'Host.Inventory.DeleteCluster',
   'Host.Inventory.EditCluster',
   'Host.Inventory.ManageClusterLifecyle',
   'Host.Inventory.MoveCluster',
   'Host.Inventory.MoveHost',
   'Host.Inventory.RemoveHostFromCluster',
   'Host.Inventory.RenameCluster',
   'Host.Local.CreateVM',
   'Host.Local.DeleteVM',
   'Host.Local.InstallAgent',
   'Host.Local.ManageUserGroups',
   'Host.Local.ReconfigVM',
   'Infraprofile.Read',
   'Infraprofile.Write',
   'IntercomNamespace.Read',
   'IntercomNamespace.Write',
   'InventoryService.Tagging.AttachTag',
   'InventoryService.Tagging.CreateCategory',
   'InventoryService.Tagging.CreateTag',
   'InventoryService.Tagging.DeleteCategory',
   'InventoryService.Tagging.DeleteTag',
   'InventoryService.Tagging.EditCategory',
   'InventoryService.Tagging.EditTag',
   'InventoryService.Tagging.ModifyUsedByForCategory',
   'InventoryService.Tagging.ModifyUsedByForTag',
   'InventoryService.Tagging.ObjectAttachable',
   'NIMBLESTORAGE.Datacenter.com.nimblestorage.hi.View',
   'NIMBLESTORAGE.Datastore.Snapshot.com.nimblestorage.hi.Create',
   'NIMBLESTORAGE.Datastore.Snapshot.com.nimblestorage.hi.Delete',
   'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.Clone',
   'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.Create',
   'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.Delete',
   'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.Edit',
   'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.Grow',
   'NIMBLESTORAGE.Datastore.com.nimblestorage.hi.View',
   'NIMBLESTORAGE.NimbleGroup.ManageGroup',
   'NIMBLESTORAGE.NimbleGroup.groupId.406793085223850253',
   'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.Claim',
   'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.ConfigureVss',
   'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.Delete',
   'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.Purge',
   'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.Restore',
   'NIMBLESTORAGE.VirtualMachine.Inventory.com.nimblestorage.hi.Undelete',
   'NIMBLESTORAGE.VirtualMachine.View.com.nimblestorage.hi.Deleted',
   'NIMBLESTORAGE.VirtualMachine.View.com.nimblestorage.hi.Local',
   'NIMBLESTORAGE.VirtualMachine.View.com.nimblestorage.hi.Replicated',
   'Namespaces.Configure',
   'Namespaces.Manage',
   'Network.Assign',
   'Network.Config',
   'Network.Delete',
   'Network.Move',
   'Nsx.Manage',
   'Nsx.ModifyAll',
   'Nsx.Read',
   'Observability.Admin',
   'Performance.ModifyIntervals',
   'Plugin.Management',
   'Profile.Clear',
   'Profile.Create',
   'Profile.Delete',
   'Profile.Edit',
   'Profile.Export',
   'Profile.View',
   'Resource.ApplyRecommendation',
   'Resource.AssignVAppToPool',
   'Resource.AssignVMToPool',
   'Resource.ColdMigrate',
   'Resource.CreatePool',
   'Resource.DeletePool',
   'Resource.EditPool',
   'Resource.HotMigrate',
   'Resource.MovePool',
   'Resource.QueryVMotion',
   'Resource.RenamePool',
   'ScheduledTask.Create',
   'ScheduledTask.Delete',
   'ScheduledTask.Edit',
   'ScheduledTask.Run',
   'ServiceAccount.Administer',
   'ServiceAccount.ManageAccount',
   'ServiceAccount.ManagePassword',
   'Sessions.GlobalMessage',
   'Sessions.ImpersonateUser',
   'Sessions.TerminateSession',
   'Sessions.ValidateSession',
   'SettingsStore.Manage',
   'StoragePod.Config',
   'StorageProfile.Update',
   'StorageProfile.View',
   'StorageViews.ConfigureService',
   'StorageViews.View',
   'System.Anonymous',
   'System.Read',
   'System.View',
   'Task.Create',
   'Task.Update',
   'TenantManager.Query',
   'TenantManager.Update',
   'TransferService.Manage',
   'TransferService.Monitor',
   'Trust.Administer',
   'Trust.Manage',
   'VApp.ApplicationConfig',
   'VApp.AssignResourcePool',
   'VApp.AssignVApp',
   'VApp.AssignVM',
   'VApp.Clone',
   'VApp.Create',
   'VApp.Delete',
   'VApp.Export',
   'VApp.ExtractOvfEnvironment',
   'VApp.Import',
   'VApp.InstanceConfig',
   'VApp.ManagedByConfig',
   'VApp.Move',
   'VApp.PowerOff',
   'VApp.PowerOn',
   'VApp.PullFromUrls',
   'VApp.Rename',
   'VApp.ResourceConfig',
   'VApp.Suspend',
   'VApp.Unregister',
   'VcIdentityProviders.Create',
   'VcIdentityProviders.Manage',
   'VcIdentityProviders.Read',
   'VcIntegrity.Baseline.com.vmware.vcIntegrity.AssignBaselines',
   'VcIntegrity.Baseline.com.vmware.vcIntegrity.ManageBaselines',
   'VcIntegrity.FileUpload.com.vmware.vcIntegrity.ImportFile',
   'VcIntegrity.General.com.vmware.vcIntegrity.Configure',
   'VcIntegrity.HardwareCompatibility.Read',
   'VcIntegrity.Updates.com.vmware.vcIntegrity.Remediate',
   'VcIntegrity.Updates.com.vmware.vcIntegrity.Scan',
   'VcIntegrity.Updates.com.vmware.vcIntegrity.Stage',
   'VcIntegrity.Updates.com.vmware.vcIntegrity.ViewStatus',
   'VcIntegrity.lifecycleGeneral.Read',
   'VcIntegrity.lifecycleGeneral.Write',
   'VcIntegrity.lifecycleHealth.Read',
   'VcIntegrity.lifecycleHealth.Write',
   'VcIntegrity.lifecycleSettings.Read',
   'VcIntegrity.lifecycleSettings.Write',
   'VcIntegrity.lifecycleSoftwareRemediation.Read',
   'VcIntegrity.lifecycleSoftwareRemediation.Write',
   'VcIntegrity.lifecycleSoftwareSpecification.Read',
   'VcIntegrity.lifecycleSoftwareSpecification.Write',
   'VcLifecycle.View',
   'VirtualMachine.Config.AddExistingDisk',
   'VirtualMachine.Config.AddNewDisk',
   'VirtualMachine.Config.AddRemoveDevice',
   'VirtualMachine.Config.AdvancedConfig',
   'VirtualMachine.Config.Annotation',
   'VirtualMachine.Config.CPUCount',
   'VirtualMachine.Config.ChangeTracking',
   'VirtualMachine.Config.DiskExtend',
   'VirtualMachine.Config.EditDevice',
   'VirtualMachine.Config.HostUSBDevice',
   'VirtualMachine.Config.ManagedBy',
   'VirtualMachine.Config.Memory',
   'VirtualMachine.Config.MksControl',
   'VirtualMachine.Config.QueryFTCompatibility',
   'VirtualMachine.Config.QueryUnownedFiles',
   'VirtualMachine.Config.RawDevice',
   'VirtualMachine.Config.ReloadFromPath',
   'VirtualMachine.Config.RemoveDisk',
   'VirtualMachine.Config.Rename',
   'VirtualMachine.Config.ResetGuestInfo',
   'VirtualMachine.Config.Resource',
   'VirtualMachine.Config.Settings',
   'VirtualMachine.Config.SwapPlacement',
   'VirtualMachine.Config.ToggleForkParent',
   'VirtualMachine.Config.UpgradeVirtualHardware',
   'VirtualMachine.GuestOperations.Execute',
   'VirtualMachine.GuestOperations.Modify',
   'VirtualMachine.GuestOperations.ModifyAliases',
   'VirtualMachine.GuestOperations.Query',
   'VirtualMachine.GuestOperations.QueryAliases',
   'VirtualMachine.Hbr.ConfigureReplication',
   'VirtualMachine.Hbr.MonitorReplication',
   'VirtualMachine.Hbr.ReplicaManagement',
   'VirtualMachine.Interact.AnswerQuestion',
   'VirtualMachine.Interact.Backup',
   'VirtualMachine.Interact.ConsoleInteract',
   'VirtualMachine.Interact.CreateScreenshot',
   'VirtualMachine.Interact.CreateSecondary',
   'VirtualMachine.Interact.DefragmentAllDisks',
   'VirtualMachine.Interact.DeviceConnection',
   'VirtualMachine.Interact.DisableSecondary',
   'VirtualMachine.Interact.DnD',
   'VirtualMachine.Interact.EnableSecondary',
   'VirtualMachine.Interact.GuestControl',
   'VirtualMachine.Interact.MakePrimary',
   'VirtualMachine.Interact.Pause',
   'VirtualMachine.Interact.PowerOff',
   'VirtualMachine.Interact.PowerOn',
   'VirtualMachine.Interact.PutUsbScanCodes',
   'VirtualMachine.Interact.Record',
   'VirtualMachine.Interact.Replay',
   'VirtualMachine.Interact.Reset',
   'VirtualMachine.Interact.SESparseMaintenance',
   'VirtualMachine.Interact.SetCDMedia',
   'VirtualMachine.Interact.SetFloppyMedia',
   'VirtualMachine.Interact.Suspend',
   'VirtualMachine.Interact.SuspendToMemory',
   'VirtualMachine.Interact.TerminateFaultTolerantVM',
   'VirtualMachine.Interact.ToolsInstall',
   'VirtualMachine.Interact.TurnOffFaultTolerance',
   'VirtualMachine.Inventory.Create',
   'VirtualMachine.Inventory.CreateFromExisting',
   'VirtualMachine.Inventory.Delete',
   'VirtualMachine.Inventory.Move',
   'VirtualMachine.Inventory.Register',
   'VirtualMachine.Inventory.Unregister',
   'VirtualMachine.Namespace.Event',
   'VirtualMachine.Namespace.EventNotify',
   'VirtualMachine.Namespace.Management',
   'VirtualMachine.Namespace.ModifyContent',
   'VirtualMachine.Namespace.Query',
   'VirtualMachine.Namespace.ReadContent',
   'VirtualMachine.Provisioning.Clone',
   'VirtualMachine.Provisioning.CloneTemplate',
   'VirtualMachine.Provisioning.CreateTemplateFromVM',
   'VirtualMachine.Provisioning.Customize',
   'VirtualMachine.Provisioning.DeployTemplate',
   'VirtualMachine.Provisioning.DiskRandomAccess',
   'VirtualMachine.Provisioning.DiskRandomRead',
   'VirtualMachine.Provisioning.FileRandomAccess',
   'VirtualMachine.Provisioning.MarkAsTemplate',
   'VirtualMachine.Provisioning.MarkAsVM',
   'VirtualMachine.Provisioning.ModifyCustSpecs',
   'VirtualMachine.Provisioning.PromoteDisks',
   'VirtualMachine.Provisioning.ReadCustSpecs',
   'VirtualMachine.State.RemoveSnapshot',
   'VirtualMachine.State.RenameSnapshot',
   'VirtualMachine.State.RevertToSnapshot',
   'Vsan.Cluster.ShallowRekey',
   'vService.CreateDependency',
   'vService.DestroyDependency',
   'vService.ReconfigureDependency',
   'vService.UpdateDependency',
   'vStats.CollectAny',
   'vStats.QueryAny',
   'vStats.Settings'
]

Process finished with exit code 0

"""